	
//  ******************      TO DO     ******************  //

print ("Welcome to the Sudoku Solver");
print ("Email comments/complaints to wingalls@ucsd.edu");

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
*	
*												Constants and look-up tables
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
var dim = 9;
var screen_size = preferences.screen_dimensions.value;
var size = parseInt(screen_size,10);
var pref_font = preferences.fontStyle.value;
var font = pref_font.replace(/(^.)* (Regular.*)|(Plain.*)/,"$1");
var bold_font = preferences.fontBold.value;
var italics_font = preferences.fontItalics.value;
var tool_tips = (preferences.tooltip.value === "0");
var border = preferences.border.value;
var hint = preferences.hint.value;
var board_color = preferences.colorStyle.value;
var board_color_on = preferences.colorOn.value;
var tile_texture = preferences.texture.value;
var fontStyle;
var superhint = (preferences.superhint.value === "1");
var difficulty = parseInt(preferences.difficulty.value, 10);
var no_help = (preferences.no_help.value === "1");
var solve_level = parseInt(preferences.solve_level.value, 10);
var no_questions = (preferences.no_questions.value === "1");
var use_special_characters = (preferences.special_character_set.value === "1");
var special_character_set = preferences.character_set.value;

var difficulty_abbr = ["Easy","Med","Hard","Ext","Easy"];
var difficulty_name = ["easy","medium","hard","extreme","easy"];
var difficulty_title = ["Easy","Medium","Hard","Extreme","Easy"];
var difficulty_title_prefixed = ["an Easy","a Medium","a Hard","an Extreme","an Easy"];
var max_x = screen.availWidth;
var max_y = screen.availHeight;
var tile_size = [60, 60, 45];
var small_font_size = [18, 18, 14];
var big_font_size = [40, 40, 30];
var small_number_model = new Text();
small_number_model.size = small_font_size[size];
small_number_model.data = (dim==12?"99":"9");
var big_number_model = new Text();
big_number_model.size = big_font_size[size];
big_number_model.data = (dim==12?"99":"9");
var recent_puzzles_array = new Array();
var recent_puzzles_array_limit = 25;

// Color pseudos //
var COLOR_RED = "#FF0000";
var COLOR_NUMBERS = "#444444";
var COLOR_BLUE = "#0000FF";
var COLOR_YELLOW = "#FFFF00";
var COLOR_GREEN = "#00FF00";
var COLOR_WHITE = "#FFFFFF";
var COLOR_BLACK = "#000000";
var COLOR_PINK = "#FF00FF";

var RAINBOW = COLOR_RED + COLOR_BLUE + COLOR_YELLOW + COLOR_GREEN + COLOR_WHITE + COLOR_BLACK + COLOR_PINK + COLOR_NUMBERS;

// Solve pseudos //
var SOLVE_NAKED_SINGLE = 1;
var SOLVE_HIDDEN_SINGLE = 2;
var SOLVE_BLOCK_RC = 3;
var SOLVE_BLOCK_BLOCK = 4;
var SOLVE_NAKED_SUBSET = 5;
var SOLVE_HIDDEN_SUBSET = 6;
var SOLVE_XWING = 7;
var SOLVE_SWORDFISH = 8;
var SOLVE_ENTIRE_PUZZLE = 9;

// Screen Size pseudos //
var SCREEN_AUTO = 0;
var SCREEN_LARGE = 1;
var SCREEN_SMALL = 2;

var tileToSquare = null;
var squareToTileI = null;
var squareToTileJ = null;

// Make hint text arrays //
var hintTextTerse = ["None.",						// 0
					"Naked Single",					// 1
					"Hidden Single",				// 2
					"Hidden Single",				// 3
					"Hidden Single",				// 4
					"Block to Col/Row Interaction",	// 5
					"Error",						// 6
					"Missing #",					// 7
					"Missing #",					// 8
					"Missing #",					// 9
					"Row to Block Interaction",		// 10
					"Column to Block Interaction",	// 11
					"Naked Subset",					// 12
					"Hidden Subset",				// 13
					"Puzzle Complete",				// 14
					"No more free ride",			// 15
					"XWing",						// 16
					"Swordfish"];					// 17
								
var hintTextVerbose = ["I can't find any. You're on your own.",
						"The only possiblity for that cell.",
						"It's the only # in that square.",
						"It's the only # in that row.",
						"It's the only # in that column.",
						"The # must be in that row/column. Erase the yellow ones.",
						"OOPS! There are no numbers in this tile.",
						"OOPS! The number is missing in this row.",
						"OOPS! The number is missing in this column.",
						"OOPS! The number is missing in this square.",
						"The # must be in that row. Erase the yellow numbers.",
						"The # must be in that column. Erase the yellow numbers.",
						"Erase the yellow numbers.",
						"Erase the yellow numbers.",
						"You silly rabbit.",
						"Buster",
						"Erase the yellow numbers.",
						"Erase the yellow numbers."];
				
var hintLinks = ["",
					"http://www.sadmansoftware.com/sudoku/technique1.htm",
					"http://www.sadmansoftware.com/sudoku/technique2.htm",
					"http://www.sadmansoftware.com/sudoku/technique2.htm",
					"http://www.sadmansoftware.com/sudoku/technique2.htm",
					"http://www.sadmansoftware.com/sudoku/technique3.htm",
					"",
					"",
					"",
					"",
					"http://www.sadmansoftware.com/sudoku/technique4.htm",
					"http://www.sadmansoftware.com/sudoku/technique4.htm",
					"http://www.sadmansoftware.com/sudoku/technique5.htm",
					"http://www.sadmansoftware.com/sudoku/technique9.htm",
					"",
					"",
					"http://www.sadmansoftware.com/sudoku/technique6.htm",
					"http://www.sadmansoftware.com/sudoku/technique13.htm"];
		
var default_candidates = "123456789ABCDEFG";				
var candidates = "123456789ABCDEFG";

// Make arrays and various global variables //
var undoList = new Array();
var tiles = null;
var bigNumbers = null;
var	smallNumbers = null;
var reset_buttonState = "closed";
var untint_button = null;

var dummyObject = new ListOfCoordinates();
dummyObject = new Coordinate();
dummyObject = null;

var squares = null;
var rows = null;
var columns = null;
var numbersR = null;
var numbersC = null;

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
*	
*														Objects 
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
function pushCoord(i, j, k){
	var coordinate = new Coordinate();
	coordinate.row = i;
	coordinate.column = j;
	coordinate.number = k;
	this.list[this.list.length] = coordinate;		
} // end pushCoord()

function popCoord(){
	var lastOfList = this.list.length-1;
	var returnValue = this.list[lastOfList];
	this.list.length = lastOfList;
	return returnValue;
}

function getSizeOfCoord(){
	return this.list.length;
}

function getCoord(index){
	return this.list[index-1];
}

function isCoordInList(i, j, k){
	for(var n=0; n<this.list.length; n++){
		var coordinate = this.list[n];
		if (coordinate.row == i){
			if (coordinate.column == j){
				if (coordinate.number == k){
					return true;
				}
			}
		}
	}
	return false;
}

function listConcat(secondList){
	var n = secondList.getSize();
	for (var x=0; x<n; x++){
		this.list[this.list.length] = secondList.getCoord(x+1);
	}
}

function coordListToString(){
	var returnValue = "Coord List: ";
	var size = this.list.length;
	for (var x=0; x<size; x++){
		returnValue = returnValue + "\n\t" + x + ": " + this.getCoord(x+1);
	}
	return returnValue;
}
function ListOfCoordinates(){
	this.list = new Array();
}
	
ListOfCoordinates.prototype.getSize = getSizeOfCoord;
ListOfCoordinates.prototype.push = pushCoord;
ListOfCoordinates.prototype.pop = popCoord;
ListOfCoordinates.prototype.getCoord = getCoord;
ListOfCoordinates.prototype.isCoordInList = isCoordInList;
ListOfCoordinates.prototype.concat = listConcat;
ListOfCoordinates.prototype.toString = coordListToString;
 // end ListOfCoordinates()

function coordToString(){
	var returnValue = "Coordinate: ";
	returnValue = returnValue + " Row: " + this.row;
	returnValue = returnValue + ", Col: " + this.column;
	returnValue = returnValue + ", Num: " + this.number;
	return returnValue;
}

function coordEqual(secondCoord){
	if (this.row == secondCoord.row){
		if (this.column == secondCoord.column){
			if (this.number == secondCoord.number){
				return true;
			}
		}
	}
	return false;		
}

function coordInSameTile(secondCoord){
	if (this.row == secondCoord.row){
		if (this.column == secondCoord.column){
			return true;
		}
	}
}

function Coordinate(){
	this.row = 0;
	this.column = 0;
	this.number = 0;
}
Coordinate.prototype.toString = coordToString;
Coordinate.prototype.isEqual = coordEqual;
Coordinate.prototype.isInSameTile = coordInSameTile;
// end object Coordinate()

function push(i, j, k){
	this.row[this.size] = i;
	this.column[this.size] = j;
	this.number[this.size] = k;
	this.size++;
}// end push()

function pop(){
	var returnValue = new Array(3);
	returnValue[0] = this.row[this.size-1];
	this.row.length = this.row.length-1;
	returnValue[1] = this.column[this.size-1];
	this.column.length = this.column.length-1;
	returnValue[2] = this.number[this.size-1];
	this.number.length = this.number.length-1;
	this.size--;
	return returnValue;
}// end pop()


function undoObject(){
	this.row = new Array();
	this.column = new Array();
	this.number = new Array();
	this.size = 0;
	this.push = push;
	this.pop = pop;			
}// end undoObject()

function setButtonText(button){
	this.hOffset = button.hOffset + Math.ceil(button.width/2);
	this.vOffset = button.vOffset + button.height - Math.ceil((button.height - this.size));
}
Text.prototype.setButtonText = setButtonText;

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
*	
*														Undo & Reset 
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

function undo(){
	if (undoList.length > 0){
		var undoSet = undoList[undoList.length-1];
		undoList.length--;
		var undoSize = undoSet.size;
		var coordinates = new Array(3);
		for(var z=0; z<undoSize; z++){
			coordinates = undoSet.pop();
			var i = coordinates[0];
			var j = coordinates[1];
			var k = coordinates[2];
			var square = tileToSquare[i][j];
			var posit = squareToTileJ[i][j];
			smallNumbers[i][j][k].data = candidates.charAt(k);
			bigNumbers[i][j].data = "";
			squares[square][k][0] = squares[square][k][0] | (1 << posit);
			squares[square][k][1]++;
			rows[i][k][0] = rows[i][k][0] | (1 << j);
			rows[i][k][1]++;
			columns[j][k][0] = columns[j][k][0] | (1 << i);
			columns[j][k][1]++;
			numbersR[k][i][0] = numbersR[k][i][0] | (1 << j);
			numbersR[k][i][1]++;
			numbersC[k][j][0] = numbersC[k][j][0] | (1 << i);
			numbersC[k][j][1]++;
		}
		if (undoList.length < 1){
			undo_text.opacity=100;
			undo_button.tooltip = "";
		}
	}
}// end undo()

function undoAll(){
	while(undoList.length > 0){
		undo();
	}
}

function reset(){
	if(reset_buttonState != "open"){
		reset_button.src="Resources/reset_lit.jpg";
		reset_lid.src="Resources/reset_lid.jpg";
		reset_buttonState = "open";
		if(tool_tips){
			reset_button.tooltip = "Click to reset puzzle.";
			reset_lid.tooltip = "Click to close lid.";
		}
		return;
	} else {
		reset_button.src="Resources/reset.jpg";
		reset_lid.src="";
		reset_buttonState = "closed";
		undoList.length = 0;
		undo_text.opacity = 100;
		if(tool_tips){
			reset_button.tooltip = "Click to open lid.";
			reset_lid.tooltip = "";
		}
		initializeBoard();
		hideHintText();
	}
}// end reset()

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
*	
*														Look and Feel 
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */


function setFont(font){
	for(var i=0; i<dim; i++){
		if(no_help){
			markupBigNumbers[i].font = font;
			markupBigNumbers[i].style = fontStyle;
			markupSmallNumbers[i].font = font;
			markupSmallNumbers[i].style = fontStyle;
		}
		for(var j=0; j<dim; j++){
			bigNumbers[i][j].font = font;
			bigNumbers[i][j].style = fontStyle;
			for (var k=0; k<dim; k++){
				smallNumbers[i][j][k].font = font;
				smallNumbers[i][j][k].style = fontStyle;
			}
		}
	}
	text1.font = font;
	text1.style = fontStyle;
	text2.font = font;
	text2.style = fontStyle;
	difficulty_text.font = font;
	difficulty_text.style = fontStyle;
	new_game_text.font = font;
	new_game_text.style = fontStyle;
	undo_text.font = font;
	undo_text.style = fontStyle;
	hint_text.font = font;
	hint_text.style = fontStyle;
}// end setFont()

function tintTile(listOfCoords, color){
	var n = listOfCoords.getSize();
	var coord = new Coordinate();
	setUntintButton();
	for (var x=0; x<n; x++){
		coord = listOfCoords.getCoord(x+1);
		var i = coord.row;
		var j = coord.column;
		tiles[i][j].colorize = color;
	}
} // end tintTile()

function tintRow(listOfCoords, color){
	var n = listOfCoords.getSize();
	var coord = new Coordinate();
	setUntintButton();
	for (var x=0; x<n; x++){
		coord = listOfCoords.getCoord(x+1);
		var i = coord.row;
		for (var j=0; j<dim; j++){
			tiles[i][j].colorize = color;
		}
	}
} // end tintRow()

function tintColumn(listOfCoords, color){
	var n = listOfCoords.getSize();
	var coord = new Coordinate();
	setUntintButton();
	for (var x=0; x<n; x++){
		coord = listOfCoords.getCoord(x+1);
		var j = coord.column;
		for (var i=0; i<dim; i++){
			tiles[i][j].colorize = color;
		}
	}
} // end tintColumn()

function tintSquare(listOfCoords, color){
	var n = listOfCoords.getSize();
	var coord = new Coordinate();
	setUntintButton();
	for (var x=0; x<n; x++){
		coord = listOfCoords.getCoord(x+1);
		var i = coord.row;
		var j = coord.column;
		var square = tileToSquare[i][j];
		for(var y=0; y<dim; y++){
			i = squareToTileI[square][y];
			j = squareToTileJ[square][y];
			tiles[i][j].colorize = color;
		}
	}
} // end tintSquare()

function tintSmallNumbers(listOfCoords, color){
	var n = listOfCoords.getSize();
	var coord = new Coordinate();
	setUntintButton();
	for (var x=0; x<n; x++){
		coord = listOfCoords.getCoord(x+1);
		smallNumbers[coord.row][coord.column][coord.number].color = color;
	}
} // end tintSmallNumbers()

function setUntintButton(){
	untint_button = new Image();
	untint_button.src = "Resources/" + tile_texture + "/button.jpg";
	untint_button.height = 10;
	untint_button.width = 10;
	untint_button.hOffset = hint_button.hOffset + hint_button.width + 5;
	untint_button.vAlign = "bottom";
	untint_button.vOffset = hint_button.vOffset + hint_button.height;
	untint_button.onMouseDown = "untintButtonPushed();";
	untint_button.onMouseUp = "untintButtonReleased();";
	if(tool_tips){
		untint_button.tooltip = "Click to restore number colors.";
	}
	if (preferences.colorOn.value > 0){
		untint_button.colorize = board_color;
	}
	var items = new Array();
	items[0] = new MenuItem();
	items[0].title = "Dump data to file";
	items[0].enabled = true;
	items[0].onSelect = "dumpDataToFile();";
	untint_button.contextMenuItems = items;
}

function untintButtonPushed(){
	untint_button.src = "Resources/" + tile_texture + "/pressed_button.jpg";
}

function untintButtonReleased(){
	restoreColors();
	hideHintText();
	untint_button = null;
}

function restoreColors(){
	uncolorSmallNumbers();
	uncolorTiles();	
}

function uncolorSmallNumbers(){
	for (var i=0; i<dim; i++){
		for (var j=0; j<dim; j++){
			for (var k=0; k<dim; k++){
				if (smallNumbers[i][j][k].data !== ""){
					smallNumbers[i][j][k].color = COLOR_NUMBERS;
				}
			}
		}
	}
}

function uncolorTiles(){
	for (var i=0; i<dim; i++){
		for (var j=0; j<dim; j++){
			if (preferences.colorOn.value > 0){
				tiles[i][j].colorize = board_color;
			} else {
				tiles[i][j].colorize = "";
			}
		}
	}
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
*	
*														Hint 
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

function parseNumberSign(text_to_search, character_number){
	var splitTextArray;
	var returnValue = text_to_search;
	splitTextArray = text_to_search.split("#");
	if (splitTextArray.length > 1){
		var newText = "";
		for (var x=0; x<splitTextArray.length-1; x++){
			newText = newText + splitTextArray[x] + candidates.charAt(character_number);
		}
		returnValue = newText + splitTextArray[splitTextArray.length-1];
	}
	return returnValue;
}

function showHintText(textIndex, k){
	var hintTextString = "";
	var extraHintString = "";
	switch(hint){
		case "Verbose":
			extraHintString = parseNumberSign(hintTextVerbose[textIndex], k);
		case "Terse":
			hintTextString =  parseNumberSign(hintTextTerse[textIndex],k);
			if(hintLinks[textIndex] !== ""){
				text1.onMouseUp = "text1OnMouseUp(\"" + hintLinks[textIndex]+ "\");";
				text1.tooltip = "Click to open browser at SadMan Software: Techniques for Solving Sudoku"
				text1.color = COLOR_BLUE;
				text1.style = "bold";
			} else {
				text1.onMouseUp = "";
				text1.tooltip = "";
				text1.color = COLOR_BLACK;
				text1.style = text1.style = fontStyle;
			}		
		default:
			text1.data = hintTextString;
			text2.data = extraHintString;
	}
	setUntintButton();
}// end showHintText()

function text1OnMouseUp(link_text){
	openURL(link_text);
}

function hideHintText(){
	text2.data = "";
	text1.data = "";
}// end hideHintText()

function solvePuzzle(blueNumbers, yellowNumbers){
	var blues = blueNumbers.getSize();
	var coord;
	var yellows;
	if (yellowNumbers === null){
		// blueNumbers = coord to choose
		for (var x=0; x<blues; x++){
			coord = blueNumbers.getCoord(x+1);
			smallNumberHandler(coord.row, coord.column, coord.number, false);
		}
		updateNow();
		showHint(true);
	} else {
		yellows = yellowNumbers.getSize();
		// yellowNumbers = coords to erase
		for (var x=0; x<yellows; x++){
			coord = yellowNumbers.getCoord(x+1);
			smallNumberHandler(coord.row, coord.column, coord.number, true);
		}
		updateNow();	
		showHint(true);
	}
}

function reverseSolvePuzzle(blueNumbers, yellowNumbers){
	var blues = blueNumbers.getSize();
	var coord;
	var yellows;
	if (yellowNumbers === null){
		// blueNumbers = coord to choose
		for (var x=0; x<blues; x++){
			coord = blueNumbers.getCoord(x+1);
			smallNumberHandler(coord.row, coord.column, coord.number, false);
		}
		updateNow();
		reverseHint(true);
	} else {
		yellows = yellowNumbers.getSize();
		// yellowNumbers = coords to erase
		for (var x=0; x<yellows; x++){
			coord = yellowNumbers.getCoord(x+1);
			smallNumberHandler(coord.row, coord.column, coord.number, true);
		}
		updateNow();	
		reverseHint(true);
	}
}

function testCompletedPuzzle(){
// return true if puzzle complete and correct
	// finished
	hintSolution = checkForCompletedPuzzle();
	if (hintSolution === null){
		return false;
	}

	// error
	hintSolution = findAnEmptyTile();
	if (hintSolution !== null){
		tintTile(hintSolution, COLOR_RED);
		coord = hintSolution.getCoord(1);
		showHintText(6, coord.number);
		return false;
	}	

	// error
	hintSolution = findAMissingNumberInARow();
	if (hintSolution !== null){
		tintRow(hintSolution, COLOR_RED);
		coord = hintSolution.getCoord(1);
		showHintText(7, coord.number);
		return false;
	}
	
	// error
	hintSolution = findAMissingNumberInAColumn();
	if (hintSolution !== null){
		tintColumn(hintSolution, COLOR_RED);
		coord = hintSolution.getCoord(1);
		showHintText(8, coord.number);
		return false;
	}
	
	// error
	hintSolution = findAMissingNumberInASquare();
	if (hintSolution !== null){
		tintSquare(hintSolution, COLOR_RED);
		coord = hintSolution.getCoord(1);
		showHintText(9, coord.number);
		return false;
	}
	
	return true;
}

function puzzleCorrect(){
	showHintText(14, 0);
}

function showHint(solve){
	var hintSolution = null;
	var blueNumbers, yellowNumbers;
	var coord;
	
	// finished
	hintSolution = checkForCompletedPuzzle();
	if (hintSolution !== null){
		showHintText(14,0);
		return;
	}

	// error
	hintSolution = findAnEmptyTile();
	if (hintSolution !== null){
		tintTile(hintSolution, COLOR_RED);
		coord = hintSolution.getCoord(1);
		showHintText(6, coord.number);
		return;
	}	

	// error
	hintSolution = findAMissingNumberInARow();
	if (hintSolution !== null){
		tintRow(hintSolution, COLOR_RED);
		coord = hintSolution.getCoord(1);
		showHintText(7, coord.number);
		return;
	}
	
	// error
	hintSolution = findAMissingNumberInAColumn();
	if (hintSolution !== null){
		tintColumn(hintSolution, COLOR_RED);
		coord = hintSolution.getCoord(1);
		showHintText(8, coord.number);
		return;
	}
	
	// error
	hintSolution = findAMissingNumberInASquare();
	if (hintSolution !== null){
		tintSquare(hintSolution, COLOR_RED);
		coord = hintSolution.getCoord(1);
		showHintText(9, coord.number);
		return;
	}
	
	// Naked Single
	hintSolution = findNakedSingle();
	if (hintSolution !== null){
		if(solve && solve_level >= SOLVE_NAKED_SINGLE){
			solvePuzzle(hintSolution, null);
		} else {
			tintSmallNumbers(hintSolution, COLOR_RED);
			coord = hintSolution.getCoord(1);
			showHintText(1, coord.number);
		}
		return;
	}
	
	// Hidden Single
	hintSolution = findHiddenSingleInASquare();
	if (hintSolution !== null){
		if(solve){
			if (solve_level >= SOLVE_HIDDEN_SINGLE){
				solvePuzzle(hintSolution, null);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(hintSolution, COLOR_RED);
			coord = hintSolution.getCoord(1);
			showHintText(2, coord.number);
		}
		return;
	}
	
	// Hidden Single
	hintSolution = findHiddenSingleInARow();
	if (hintSolution !== null){
		if(solve){
			if (solve_level >= SOLVE_HIDDEN_SINGLE){
				solvePuzzle(hintSolution, null);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(hintSolution, COLOR_RED);
			coord = hintSolution.getCoord(1);
			showHintText(3, coord.number);
		}
		return;
	}
	
	// Hidden Single
	hintSolution = findHiddenSingleInAColumn();
	if (hintSolution !== null){
		if(solve){
			if (solve_level >= SOLVE_HIDDEN_SINGLE){
				solvePuzzle(hintSolution, null);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(hintSolution, COLOR_RED);
			coord = hintSolution.getCoord(1);
			showHintText(4, coord.number);
		}
		return;
	}
	
	// Block to Row/Col
	hintSolution = findBlockToRowOrCol();
	if (hintSolution !== null){
		blueNumbers = hintSolution[0];
		yellowNumbers = hintSolution[1];
		if(solve){
			if (solve_level >= SOLVE_BLOCK_RC){
				solvePuzzle(blueNumbers, yellowNumbers);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(blueNumbers, COLOR_BLUE);
			tintSmallNumbers(yellowNumbers, COLOR_YELLOW);
			showHintText(5,hintSolution[0].getCoord(1).number);
		}
		return;
	}
	
	// Block Block
	hintSolution = findRowToBlock();
	if (hintSolution !== null){
		blueNumbers = hintSolution[0];
		yellowNumbers = hintSolution[1];
		if(solve){
			if (solve_level >= SOLVE_BLOCK_BLOCK){
				solvePuzzle(blueNumbers, yellowNumbers);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(blueNumbers, COLOR_BLUE);
			tintSmallNumbers(yellowNumbers, COLOR_YELLOW);
			showHintText(10,hintSolution[0].getCoord(1).number);
		}
		return;
	}
	
	// Block Block
	hintSolution = findColumnToBlock();
	if (hintSolution !== null){
		blueNumbers = hintSolution[0];
		yellowNumbers = hintSolution[1];
		if(solve){
			if (solve_level >= SOLVE_BLOCK_BLOCK){
				solvePuzzle(blueNumbers, yellowNumbers);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(blueNumbers, COLOR_BLUE);
			tintSmallNumbers(yellowNumbers, COLOR_YELLOW);
			showHintText(11,hintSolution[0].getCoord(1).number);
		}
		return;
	}
	
	// Naked Subset
	hintSolution = findNakedPairInASquare();
	if (hintSolution !== null){
		blueNumbers = hintSolution[0];
		yellowNumbers = hintSolution[1];
		if(solve){
			if (solve_level >= SOLVE_NAKED_SUBSET){
				solvePuzzle(blueNumbers, yellowNumbers);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(blueNumbers, COLOR_BLUE);
			tintSmallNumbers(yellowNumbers, COLOR_YELLOW);
			showHintText(12,0);
		}
		return;
	}
	
	// Naked Subset
	hintSolution = findNakedPairInARow();
	if (hintSolution !== null){
		blueNumbers = hintSolution[0];
		yellowNumbers = hintSolution[1];
		if(solve){
			if (solve_level >= SOLVE_NAKED_SUBSET){
				solvePuzzle(blueNumbers, yellowNumbers);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(blueNumbers, COLOR_BLUE);
			tintSmallNumbers(yellowNumbers, COLOR_YELLOW);
			showHintText(12,0);
		}
		return;
	}
	
	// Naked Subset
	hintSolution = findNakedPairInAColumn();
	if (hintSolution !== null){
		blueNumbers = hintSolution[0];
		yellowNumbers = hintSolution[1];
		if(solve){
			if (solve_level >= SOLVE_NAKED_SUBSET){
				solvePuzzle(blueNumbers, yellowNumbers);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(blueNumbers, COLOR_BLUE);
			tintSmallNumbers(yellowNumbers, COLOR_YELLOW);
			showHintText(12,0);
		}
		return;
	}
	
	// Hidden Subset
	hintSolution = findHiddenSubsetsInSquares();
	if (hintSolution !== null){
		blueNumbers = hintSolution[0];
		yellowNumbers = hintSolution[1];
		if(solve){
			if (solve_level >= SOLVE_HIDDEN_SUBSET){
				solvePuzzle(blueNumbers, yellowNumbers);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(blueNumbers, COLOR_BLUE);
			tintSmallNumbers(yellowNumbers, COLOR_YELLOW);
			showHintText(13,0);
		}
		return;
	}
	
	// Hidden Subset
	hintSolution = findHiddenSubsetsInRows();
	if (hintSolution !== null){
		blueNumbers = hintSolution[0];
		yellowNumbers = hintSolution[1];
		if(solve){
			if (solve_level >= SOLVE_HIDDEN_SUBSET){
				solvePuzzle(blueNumbers, yellowNumbers);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(blueNumbers, COLOR_BLUE);
			tintSmallNumbers(yellowNumbers, COLOR_YELLOW);
			showHintText(13,0);
		}
		return;
	}
	
	// Hidden Subset
	hintSolution = findHiddenSubsetsInColumns();
	if (hintSolution !== null){
		blueNumbers = hintSolution[0];
		yellowNumbers = hintSolution[1];
		if(solve){
			if (solve_level >= SOLVE_HIDDEN_SUBSET){
				solvePuzzle(blueNumbers, yellowNumbers);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(blueNumbers, COLOR_BLUE);
			tintSmallNumbers(yellowNumbers, COLOR_YELLOW);
			showHintText(13,0);
		}
		return;
	}
	
	// X-Wing
	hintSolution = findXWingInRows();
	if (hintSolution !== null){
		blueNumbers = hintSolution[0];
		yellowNumbers = hintSolution[1];
		if(solve){
			if (solve_level >= SOLVE_XWING){
				solvePuzzle(blueNumbers, yellowNumbers);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(blueNumbers, COLOR_BLUE);
			tintSmallNumbers(yellowNumbers, COLOR_YELLOW);
			showHintText(16,0);
		}
		return;
	}
	
	// X-Wing
	hintSolution = findXWingInColumns();
	if (hintSolution !== null){
		blueNumbers = hintSolution[0];
		yellowNumbers = hintSolution[1];
		if(solve){
			if (solve_level >= SOLVE_XWING){
				solvePuzzle(blueNumbers, yellowNumbers);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(blueNumbers, COLOR_BLUE);
			tintSmallNumbers(yellowNumbers, COLOR_YELLOW);
			showHintText(16,0);
		}
		return;
	}
	
	// Swordfish
	hintSolution = findSwordfishInColumns();
	if (hintSolution !== null){
		blueNumbers = hintSolution[0];
		yellowNumbers = hintSolution[1];
		if(solve){
			if (solve_level >= SOLVE_SWORDFISH){
				solvePuzzle(blueNumbers, yellowNumbers);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(blueNumbers, COLOR_BLUE);
			tintSmallNumbers(yellowNumbers, COLOR_YELLOW);
			showHintText(17,0);
		}
		return;
	}

	// Swordfish
	hintSolution = findSwordfishInRows();
	if (hintSolution !== null){
		blueNumbers = hintSolution[0];
		yellowNumbers = hintSolution[1];
		if(solve){
			if (solve_level >= SOLVE_SWORDFISH){
				solvePuzzle(blueNumbers, yellowNumbers);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(blueNumbers, COLOR_BLUE);
			tintSmallNumbers(yellowNumbers, COLOR_YELLOW);
			showHintText(17,0);
		}
		return;
	}

	// None found
	showHintText(0);
}// end showHint()

function checkForUnstartedPuzzle(){
	var returnValue = new ListOfCoordinates();
	for (var i=0; i<dim; i++){
		for (var j=0; j<dim; j++){
			for (var k=0; k<dim; k++){
				if(smallNumbers[i][j][k].data === ""){
					return null;
				}
			}
		}
	}
	return returnValue;
}

function checkForCompletedPuzzle(){
	var returnValue = new ListOfCoordinates();
	for (var i=0; i<dim; i++){
		for (var j=0; j<dim; j++){
			if(bigNumbers[i][j].data === ""){
				return null;
			}
		}
	}
	return returnValue;
}

function findAnEmptyTile(){
	var returnValue = null;
	for (var i=0; i<dim; i++){
		for (var j=0; j<dim; j++){
			if (bigNumbers[i][j].data === ""){
				var foundOne = dim;
				for (var k=0; k<dim; k++){
					if (smallNumbers[i][j][k].data !== ""){
							foundOne = k;
							break;
					}
				}
				if (foundOne == dim){
					returnValue = new ListOfCoordinates();
					returnValue.push(i, j, k);
					return returnValue;
				}
			}
		}
	}
	return returnValue;
} // end findAnEmptyTile()

function findAMissingNumberInASquare(){
	var returnValue = null;
	for(var square=0; square<dim; square++){
		for (var k=0; k<dim; k++){
			var foundOne = dim;
			for (var x=0; x<dim; x++){
				var i = squareToTileI[square][x];
				var j = squareToTileJ[square][x];
				if (smallNumbers[i][j][k].data !== ""){
						foundOne = k;
						break;
				}
				if (bigNumbers[i][j].data == candidates.charAt(k)){
						foundOne = k;
						break;
				}
			}
			
			if (foundOne == dim){
				returnValue = new ListOfCoordinates();
				returnValue.push(i, j, k);
				return returnValue;
			}
		}
	}
	return returnValue;
} // end findAMissingNumberInASquare()

function findAMissingNumberInAColumn(){
	var returnValue = null;
	for (var j=0; j<dim; j++){
		for (var k=0; k<dim; k++){
			var foundOne = dim;
			for (var i=0; i<dim; i++){
				if (smallNumbers[i][j][k].data !== ""){
						foundOne = k;
						break;
				}
				if (bigNumbers[i][j].data == candidates.charAt(k)){
						foundOne = k;
						break;
				}
			}
			
			if (foundOne == dim){
				returnValue = new ListOfCoordinates();
				returnValue.push(i, j, k);
				return returnValue;
			}
		}
	}
	return returnValue;
} // end findAMissingNumberInAColumn()

function findAMissingNumberInARow(){
	var returnValue = null;
	for (var i=0; i<dim; i++){
		for (var k=0; k<dim; k++){
			var foundOne = dim;
			for (var j=0; j<dim; j++){
				if (smallNumbers[i][j][k].data !== ""){
						foundOne = k;
						break;
				}
				if (bigNumbers[i][j].data == candidates.charAt(k)){
						foundOne = k;
						break;
				}
			}
			
			if (foundOne == dim){
				returnValue = new ListOfCoordinates();
				returnValue.push(i, j, k);
				return returnValue;
			}
		}
	}
	return returnValue;
} // end findAMissingNumberInARow()

// Find the first instance of a small number alone in a tile //
function findNakedSingle(){
	var returnValue = null;
	for (var i=0; i<dim; i++){
		for (var j=0; j<dim; j++){
			if (bigNumbers[i][j].data === ""){
				var foundOne = dim;
				for (var k=0; k<dim; k++){
					if (smallNumbers[i][j][k].data !== ""){
						if (foundOne == dim){
							foundOne = k;
						} else {
							foundOne = dim+1;
							break;
						}
					}
				}
				if (foundOne < dim){
					returnValue = new ListOfCoordinates();
					returnValue.push(i,j,foundOne);
					return returnValue;
				}
			}
		}
	}
	return returnValue;
}// end findNakedSingle()

// Find the first instance of the only small number to be found in a square //
function findHiddenSingleInASquare(){
	returnValue = null;
	for(var square=0; square<dim; square++){
		for(var number=0; number<dim; number++){
			var foundOne = dim;
			var foundOneI = dim;
			var foundOneJ = dim;
			for(var tile=0; tile < dim; tile++){
				var i = squareToTileI[square][tile];
				var j = squareToTileJ[square][tile];
				var k = number;
				if (bigNumbers[i][j].data === ""){
					if (smallNumbers[i][j][k].data !== ""){
						if (foundOne == dim){
							foundOne = k;
							foundOneI = i;
							foundOneJ = j;
						} else {
							foundOne = dim+1;
							break;
						}
					}
				}
			}
			if (foundOne < dim){
				returnValue = new ListOfCoordinates();
				returnValue.push(foundOneI, foundOneJ, foundOne);
				return returnValue;
			}
		}
	}
	return returnValue;
}// end findHiddenSingleInASquare()

// Find the first instance of the only small number to be found in a row //
function findHiddenSingleInARow(){
	returnValue = null;
	for(var row=0; row<dim; row++){
		for(var number=0; number<dim; number++){
			var foundOne = dim;
			var foundOneI = dim;
			var foundOneJ = dim;
			for(var tile=0; tile < dim; tile++){
				var i = row;
				var j = tile;
				var k = number;
				if (bigNumbers[i][j].data === ""){
					if (smallNumbers[i][j][k].data !== ""){
						if (foundOne == dim){
							foundOne = k;
							foundOneI = i;
							foundOneJ = j;
						} else {
							foundOne = dim+1;
							break;
						}
					}
				}
			}
			if (foundOne < dim){
				returnValue = new ListOfCoordinates();
				returnValue.push(foundOneI, foundOneJ, foundOne);
				return returnValue;
			}
		}
	}
	return returnValue;
}// end findHiddenSingleInARow()

// Find the first instance of the only small number to be found in a column //
function findHiddenSingleInAColumn(){
	returnValue = null;
	for(var column=0; column<dim; column++){
		for(var number=0; number<dim; number++){
			var foundOne = dim;
			var foundOneI = dim;
			var foundOneJ = dim;
			for(var tile=0; tile < dim; tile++){
				var i = tile;
				var j = column;
				var k = number;
				if (bigNumbers[i][j].data === ""){
					if (smallNumbers[i][j][k].data !== ""){
						if (foundOne == dim){
							foundOne = k;
							foundOneI = i;
							foundOneJ = j;
						} else {
							foundOne = dim+1;
							break;
						}
					}
				}
			}
			if (foundOne < dim){
				returnValue = new ListOfCoordinates();
				returnValue.push(foundOneI, foundOneJ, foundOne);
				return returnValue;
			}
		}
	}
	return returnValue;
} // end findHiddenSingleInAColumn()

function findBlockToRowOrCol(){
	for(var square=0; square<dim; square++){
		for(var number=0; number<dim; number++){
			var listOfNumbers = new ListOfCoordinates();
			var listOfErasableNumbers = new ListOfCoordinates();
			var returnValue = [listOfNumbers, listOfErasableNumbers];

			for(var tile=0; tile<dim; tile++){
				var i = squareToTileI[square][tile];
				var j = squareToTileJ[square][tile];
				var k = number;
				if (bigNumbers[i][j].data == candidates.charAt(k)){
					break;
				} else {
					if (smallNumbers[i][j][k].data !== ""){
						listOfNumbers.push(i,j,k);
					}
				}
			} // end for each tile in a square
			
			// test for row condition, get erasables, and return or continue
			var n = listOfNumbers.getSize();
			var targetRow = dim;
			for(var x=0; x<n; x++){
				var coord = listOfNumbers.getCoord(x+1);
				if (targetRow == dim){
					targetRow = coord.row;
				} else {
					if (targetRow != coord.row){
						targetRow = dim;
						break;
					}
				}
			} 
			if (targetRow != dim){
				// find others in the same row
				var k = listOfNumbers.getCoord(1).number;
				var i = targetRow;
				for (var j=0; j<dim; j++){
					if (!(listOfNumbers.isCoordInList(i, j, k))){
						if (smallNumbers[i][j][k].data !== ""){
							listOfErasableNumbers.push(i, j, k);
						}
					}
				}
				if (listOfErasableNumbers.getSize() > 0){
					return returnValue;
				}
			}
			
			// test for column condition, get erasables and return, or continue
			var n = listOfNumbers.getSize();
			var targetCol = dim;
			for(var x=0; x<n; x++){
				var coord = listOfNumbers.getCoord(x+1);
				if (targetCol == dim){
					targetCol = coord.column;
				} else {
					if (targetCol != coord.column){
						targetCol = dim;
						break;
					}
				}
			} 
			if (targetCol != dim){
				// find others in the same row
				var k = listOfNumbers.getCoord(1).number;
				var j = targetCol;
				for (var i=0; i<dim; i++){
					if (!(listOfNumbers.isCoordInList(i, j, k))){
						if (smallNumbers[i][j][k].data !== ""){
							listOfErasableNumbers.push(i, j, k);
						}
					}
				}
				if (listOfErasableNumbers.getSize() > 0){
					return returnValue;
				}
			}

		} // end for each number within a square
	} // end for each square
	return null;
} // end findANumberInASquareThatExistsOnlyInOneRowOrCol()

function findRowToBlock(){
	for(var row=0; row<dim; row++){
		for(var number=0; number<dim; number++){
			var listOfNumbers = new ListOfCoordinates();
			var listOfErasableNumbers = new ListOfCoordinates();
			var returnValue = [listOfNumbers, listOfErasableNumbers];

			for(var tile=0; tile<dim; tile++){
				var i = row;
				var j = tile;
				var k = number;
				if (bigNumbers[i][j].data == candidates.charAt(k)){
					break;
				} else {
					if (smallNumbers[i][j][k].data !== ""){
						listOfNumbers.push(i,j,k);
					}
				}
			} // end for each tile in a row

			// test for square condition, get erasables, and return or continue
			var n = listOfNumbers.getSize();
			var targetSquare = dim;
			for(var x=0; x<n; x++){
				var coord = listOfNumbers.getCoord(x+1);
				if (targetSquare == dim){
					targetSquare = tileToSquare[coord.row][coord.column];
				} else {
					if (targetSquare != tileToSquare[coord.row][coord.column]){
						targetSquare = dim;
						break;
					}
				}
			} 
			if (targetSquare != dim){
				// find others in the same square
				var k = listOfNumbers.getCoord(1).number;
				for (var x=0; x<dim; x++){
					var i = squareToTileI[targetSquare][x];
					var j = squareToTileJ[targetSquare][x];
					if (!(listOfNumbers.isCoordInList(i, j, k))){
						if (smallNumbers[i][j][k].data !== ""){
							listOfErasableNumbers.push(i, j, k);
						}
					}
				}
				if (listOfErasableNumbers.getSize() > 0){
					return returnValue;
				}
			}
			
		} // end for each number within a square
	} // end for each square
	return null;
} // end findANumberInARowThatExistsOnlyInOneSquare()

function findColumnToBlock(){
	for(var col=0; col<dim; col++){
		for(var number=0; number<dim; number++){
			var listOfNumbers = new ListOfCoordinates();
			var listOfErasableNumbers = new ListOfCoordinates();
			var returnValue = [listOfNumbers, listOfErasableNumbers];

			for(var tile=0; tile<dim; tile++){
				var i = tile;
				var j = col;
				var k = number;
				if (bigNumbers[i][j].data == candidates.charAt(k)){
					break;
				} else {
					if (smallNumbers[i][j][k].data !== ""){
						listOfNumbers.push(i,j,k);
					}
				}
			} // end for each tile in a column
			
			// test for square condition, get erasables, and return or continue
			var n = listOfNumbers.getSize();
			var targetSquare = dim;
			for(var x=0; x<n; x++){
				var coord = listOfNumbers.getCoord(x+1);
				if (targetSquare == dim){
					targetSquare = tileToSquare[coord.row][coord.column];
				} else {
					if (targetSquare != tileToSquare[coord.row][coord.column]){
						targetSquare = dim;
						break;
					}
				}
			} 
			if (targetSquare != dim){
				// find others in the same square
				var k = listOfNumbers.getCoord(1).number;
				for (var x=0; x<dim; x++){
					var i = squareToTileI[targetSquare][x];
					var j = squareToTileJ[targetSquare][x];
					if (!(listOfNumbers.isCoordInList(i, j, k))){
						if (smallNumbers[i][j][k].data !== ""){
							listOfErasableNumbers.push(i, j, k);
						}
					}
				}
				if (listOfErasableNumbers.getSize() > 0){
					return returnValue;
				}
			}
			
		} // end for each number within a square
	} // end for each square
	return null;
} // end findANumberInAColumnThatExistsOnlyInOneSquare()

function findNakedPairInASquare(){
	var listOfNumbers = new ListOfCoordinates();
	var listOfErasableNumbers = new ListOfCoordinates();
	var returnValue = [listOfNumbers, listOfErasableNumbers];

	// search each square
	for(var square=0; square<dim; square++){
		var lonelies = new ListOfCoordinates();
		var allOthers = new ListOfCoordinates();
		// search each tile in a square
		for(var tile=0; tile<dim; tile++){
			var i = squareToTileI[square][tile];
			var j = squareToTileJ[square][tile];
			if (bigNumbers[i][j].data !== ""){
				continue;
			}
			var smallNumbs = new ListOfCoordinates();
			for (var k=0; k<dim; k++){
				if (smallNumbers[i][j][k].data !== ""){
					smallNumbs.push(i,j,k);
				}
			}
			// keep the lonely pairs 
			if (smallNumbs.getSize() == 2){
				lonelies.concat(smallNumbs);
			}
			allOthers.concat(smallNumbs);
		}
		// check for double pairs
		var n = lonelies.getSize();
		if (n < 4){
			continue;
		}
		for(var z=0; z<n; z=z+2){
			var coordA = lonelies.pop();
			var coordB = lonelies.pop();
			var remaining = lonelies.getSize();
			for(var y=0; y<remaining; y=y+2){
				var compA = lonelies.getCoord(y+2);
				var compB = lonelies.getCoord(y+1);
				if (coordA.number == compA.number && coordB.number == compB.number){
					// look for other tiles with these numbers
					var others = allOthers.getSize();
					for (var v=0; v<others; v++){
						var otherCoord = allOthers.getCoord(v+1);
						// skip if same tile as lonlies
						if (coordA.isInSameTile(otherCoord) || compA.isInSameTile(otherCoord)){
							continue;
						}
						if (otherCoord.number == coordA.number || otherCoord.number == coordB.number){
							listOfErasableNumbers.push(otherCoord.row, otherCoord.column, otherCoord.number);
						}
					}
				
					if (listOfErasableNumbers.getSize() > 0){
						listOfNumbers.push(coordA.row, coordA.column, coordA.number);
						listOfNumbers.push(coordB.row, coordB.column, coordB.number);
						listOfNumbers.push(compA.row, compA.column, compA.number);
						listOfNumbers.push(compB.row, compB.column, compB.number);
						return returnValue;
					}
				}
			
			}
		}
	}
	return null;
} // end findDoubleLonelyPairsInASquare()

function findNakedPairInARow(){
	var listOfNumbers = new ListOfCoordinates();
	var listOfErasableNumbers = new ListOfCoordinates();
	var returnValue = [listOfNumbers, listOfErasableNumbers];

	// search each row
	for(var row=0; row<dim; row++){
		var lonelies = new ListOfCoordinates();
		var allOthers = new ListOfCoordinates();
		// search each tile in a row
		for(var tile=0; tile<dim; tile++){
			var i = row;
			var j = tile;
			if (bigNumbers[i][j].data !== ""){
				continue;
			}
			var smallNumbs = new ListOfCoordinates();
			for (var k=0; k<dim; k++){
				if (smallNumbers[i][j][k].data !== ""){
					smallNumbs.push(i,j,k);
				}
			}
			// keep the lonely pairs 
			if (smallNumbs.getSize() == 2){
				lonelies.concat(smallNumbs);
			}
			allOthers.concat(smallNumbs);
		}
		// check for double pairs
		var n = lonelies.getSize();
		if (n < 4){
			continue;
		}
		for(var z=0; z<n; z=z+2){
			var coordA = lonelies.pop();
			var coordB = lonelies.pop();
			var remaining = lonelies.getSize();
			for(var y=0; y<remaining; y=y+2){
				var compA = lonelies.getCoord(y+2);
				var compB = lonelies.getCoord(y+1);
				if (coordA.number == compA.number && coordB.number == compB.number){
					// look for other tiles with these numbers
					var others = allOthers.getSize();
					for (var v=0; v<others; v++){
						var otherCoord = allOthers.getCoord(v+1);
						// skip if same tile as lonlies
						if (coordA.isInSameTile(otherCoord) || compA.isInSameTile(otherCoord)){
							continue;
						}

						if (otherCoord.number == coordA.number || otherCoord.number == coordB.number){
							listOfErasableNumbers.push(otherCoord.row, otherCoord.column, otherCoord.number);
						}
					}
				
					if (listOfErasableNumbers.getSize() > 0){
						listOfNumbers.push(coordA.row, coordA.column, coordA.number);
						listOfNumbers.push(coordB.row, coordB.column, coordB.number);
						listOfNumbers.push(compA.row, compA.column, compA.number);
						listOfNumbers.push(compB.row, compB.column, compB.number);
						return returnValue;
					}
				}
			
			}
		}
	}
	return null;
} // end findDoubleLonelyPairsInARow()

function findNakedPairInAColumn(){
	var listOfNumbers = new ListOfCoordinates();
	var listOfErasableNumbers = new ListOfCoordinates();
	var returnValue = [listOfNumbers, listOfErasableNumbers];

	// search each square
	for(var col=0; col<dim; col++){
		var lonelies = new ListOfCoordinates();
		var allOthers = new ListOfCoordinates();
		// search each tile in a square
		for(var tile=0; tile<dim; tile++){
			var i = tile;
			var j = col;
			if (bigNumbers[i][j].data !== ""){
				continue;
			}
			var smallNumbs = new ListOfCoordinates();
			for (var k=0; k<dim; k++){
				if (smallNumbers[i][j][k].data !== ""){
					smallNumbs.push(i,j,k);
				}
			}
			// keep the lonely pairs 
			if (smallNumbs.getSize() == 2){
				lonelies.concat(smallNumbs);
			}
			allOthers.concat(smallNumbs);
		}
		// check for double pairs
		var n = lonelies.getSize();
		if (n < 4){
			continue;
		}
		for(var z=0; z<n; z=z+2){
			var coordA = lonelies.pop();
			var coordB = lonelies.pop();
			var remaining = lonelies.getSize();
			for(var y=0; y<remaining; y=y+2){
				var compA = lonelies.getCoord(y+2);
				var compB = lonelies.getCoord(y+1);
				if (coordA.number == compA.number && coordB.number == compB.number){
					// look for other tiles with these numbers
					var others = allOthers.getSize();
					for (var v=0; v<others; v++){
						var otherCoord = allOthers.getCoord(v+1);
						// skip if same tile as lonlies
						if (coordA.isInSameTile(otherCoord) || compA.isInSameTile(otherCoord)){
							continue;
						}

						if (otherCoord.number == coordA.number || otherCoord.number == coordB.number){
							listOfErasableNumbers.push(otherCoord.row, otherCoord.column, otherCoord.number);
						}
					}
				
					if (listOfErasableNumbers.getSize() > 0){
						listOfNumbers.push(coordA.row, coordA.column, coordA.number);
						listOfNumbers.push(coordB.row, coordB.column, coordB.number);
						listOfNumbers.push(compA.row, compA.column, compA.number);
						listOfNumbers.push(compB.row, compB.column, compB.number);
						return returnValue;
					}
				}
			
			}
		}
	}
	return null;
} // end findDoubleLonelyPairsInAColumn()

var pools;
var combos;

function getCombos(poolsize, groupsize){
	combos = new Array();
	var pool = new Array();
	combinations(pool, 1, poolsize, 1, groupsize);
}

function combinations(v, start, poolsize, next, groupsize){
	if (next > groupsize) {
	    // insert code here to use combinations as you please 
		comboI = combos.length;
		combos[comboI] = new Array();
	    for (var i=1; i<=groupsize; i++){
			combos[comboI][i-1] = v[i]-1;
		}
	    return;
	}
	
	// for this k'th element of the v, try all start..n
	// elements in that position
	for (var i=start; i<=poolsize; i++) {
	    v[next] = i;
	    // recursively generate combinations of integers
	    // from i+1..n
	    combinations(v, i+1, poolsize, next+1, groupsize);
	}
}

function findHiddenSubsetsInSquares(){
	var listOfNumbers = new ListOfCoordinates();
	var listOfErasableNumbers = new ListOfCoordinates();
	var returnValue = [listOfNumbers, listOfErasableNumbers];
	for (var square=0; square<dim; square++){
		// get pools
		pools = new Array(dim);
		for(var p=0; p<dim; p++){
			pools[p] = new Array();
		}
		for(var x=0; x<dim; x++){
			var count = squares[square][x][1];
			if (count != 0){
				for(var c=count-1; c<dim; c++){
					pools[c][pools[c].length] = x; 
				}
			}
		}
		for (var r=2; r<dim; r++){ // start with doubles, repeat for all subsets
			target = new Array(r);
			indices = new Array(r);
			start = 0;
			
			pool = pools[r-1].length;
			if (pool < r){
				continue;
			}
			// get each combination of r out of pool
			// and put them in global variable combos
			getCombos(pool, r);

			for(var c=0; c<combos.length; c++){
				// or together r pools & count bits, look for r bits
				ord = 0;
				for(var b=0; b<r; b++){
					ord = ord | squares[square][pools[r-1][combos[c][b]]][0];
				}
				if (countBits(ord) == r){
					// found subset, check for erasables
					var match = new Array();
					for (f=0; f<r; f++){
						match[f] = pools[r-1][combos[c][f]];
					}
					returnValue = getSubsetValuesInSquare(square, ord, match);
					if (returnValue !== null){
						return returnValue;
					}
				}
			}

		}
	}
	return null;
} // end findHiddenSubsetsInSquare

function getSubsetValuesInSquare(square,ord,matches){
	var listOfNumbers = new ListOfCoordinates();
	var listOfErasableNumbers = new ListOfCoordinates();
	var returnValue = [listOfNumbers, listOfErasableNumbers];
	var tileCoords = new Array();
	var tmpOrd = ord;
	
	// get all of the coordinates for the blue numbers
	var match = new Array();
	for(var m=0; m<matches.length; m++){
		match[m] = squares[square][matches[m]][0];
	}
	for(var m=0; m<matches.length; m++){
		for(var b=0; b<dim; b++){
			if(match[m] %2 == 1){
				listOfNumbers.push(squareToTileI[square][b], squareToTileJ[square][b], matches[m]);
			}
			match[m] = match[m] >>> 1;
		}
	}
	// get all of the coordinates for the yellow numbers
	// get coords for tiles
	for(var b=0; b < dim; b++){
		if(tmpOrd % 2 == 1){
			var next = tileCoords.length;
			tileCoords[next]= new Coordinate();
			tileCoords[next].row = squareToTileI[square][b];
			tileCoords[next].column = squareToTileJ[square][b];
		}
		tmpOrd = tmpOrd >>> 1;
	}
	// check for other numbers in those tiles
	for (var v=0; v<dim; v++){
		var skip = false;
		for (m=0; m<matches.length; m++){
			if (v == matches[m]){
				skip = true;
				break;
			}
		}
		if (skip){
			continue;
		}
		for (m=0; m<tileCoords.length; m++){
			if (smallNumbers[tileCoords[m].row][tileCoords[m].column][v].data !== ""){
				listOfErasableNumbers.push(tileCoords[m].row, tileCoords[m].column, v);
			}
		}
	}
	
	if (listOfErasableNumbers.getSize() > 0){								
		return returnValue;
	}
	return null;
} // end getSubsetValuesInSquare()

function findHiddenSubsetsInRows(){
	var listOfNumbers = new ListOfCoordinates();
	var listOfErasableNumbers = new ListOfCoordinates();
	var returnValue = [listOfNumbers, listOfErasableNumbers];
	for (var row=0; row<dim; row++){
		// get pools
		pools = new Array(dim);
		for(var p=0; p<dim; p++){
			pools[p] = new Array();
		}

		for(var x=0; x<dim; x++){
			var count = rows[row][x][1];
			if (count != 0){
				for(var c=count-1; c<dim; c++){
					pools[c][pools[c].length] = x; 
				}
			}
		}
		for (var r=2; r<dim; r++){ // start with doubles, repeat for all subsets
			target = new Array(r);
			indices = new Array(r);
			start = 0;
			
			pool = pools[r-1].length;
			if (pool < r){
				continue;
			}
			// get each combination of r out of pool
			// and put them in global variable combos
			getCombos( pool, r);
			for(var c=0; c<combos.length; c++){
				// or together r pools & count bits, look for r bits
				ord = 0;
				for(var b=0; b<r; b++){
					ord = ord | rows[row][pools[r-1][combos[c][b]]][0];
				}
				if (countBits(ord) == r){
					// found subset, check for erasables
					var match = new Array();
					for (f=0; f<r; f++){
						match[f] = pools[r-1][combos[c][f]];
					}
					returnValue = getSubsetValuesInRow(row, ord, match);
					if (returnValue !== null){
						return returnValue;
					}
				}
			}
		}
	}
	return null;
} // end findHiddenSubsetsInRows()

function getSubsetValuesInRow(row,ord,matches){
	var listOfNumbers = new ListOfCoordinates();
	var listOfErasableNumbers = new ListOfCoordinates();
	var returnValue = [listOfNumbers, listOfErasableNumbers];
	var tileCoords = new Array();
	var tmpOrd = ord;
	
	// get all of the coordinates for the blue numbers
	var match = new Array();
	for(var m=0; m<matches.length; m++){
		match[m] = rows[row][matches[m]][0];
	}
	for(var m=0; m<matches.length; m++){
		for(var b=0; b<dim; b++){
			if(match[m] %2 == 1){
				listOfNumbers.push(row, b, matches[m]);
			}
		match[m] = match[m] >>> 1;
		}		
	}
	// get all of the coordinates for the yellow numbers
	// get coords for tiles
	for(var b=0; b < dim; b++){
		if(tmpOrd % 2 == 1){
			var next = tileCoords.length;
			tileCoords[next]= new Coordinate();
			tileCoords[next].row = row;
			tileCoords[next].column = b;
		}
		tmpOrd = tmpOrd >>> 1;
	}
	// check for other numbers in those tiles
	for (var v=0; v<dim; v++){
		var skip = false;
		for (m=0; m<matches.length; m++){
			if (v == matches[m]){
				skip = true;
				break;
			}
		}
		if (skip){
			continue;
		}
		for (m=0; m<tileCoords.length; m++){
			if (smallNumbers[tileCoords[m].row][tileCoords[m].column][v].data !== ""){
				listOfErasableNumbers.push(tileCoords[m].row, tileCoords[m].column, v);
			}
		}
	}
	
	if (listOfErasableNumbers.getSize() > 0){								
		return returnValue;
	}
	return null;
} // end getSubsetValuesInRow()

function findHiddenSubsetsInColumns(){
	var listOfNumbers = new ListOfCoordinates();
	var listOfErasableNumbers = new ListOfCoordinates();
	var returnValue = [listOfNumbers, listOfErasableNumbers];
	for (var column=0; column<dim; column++){
		// get pools
		pools = new Array(dim);
		for(var p=0; p<dim; p++){
			pools[p] = new Array();
		}

		for(var x=0; x<dim; x++){
			var count = columns[column][x][1];
			if (count != 0){
				for(var c=count-1; c<dim; c++){
					pools[c][pools[c].length] = x; 
				}
			}
		}

		for (var r=2; r<dim; r++){ // start with doubles, repeat for all subsets
			target = new Array(r);
			indices = new Array(r);
			start = 0;
			
			pool = pools[r-1].length;
			if (pool < r){
				continue;
			}
			// get each combination of r out of pool
			// and put them in global variable combos
			getCombos(pool, r);
			for(var c=0; c<combos.length; c++){
				// or together r pools & count bits, look for r bits
				ord = 0;
				for(var b=0; b<r; b++){
					ord = ord | columns[column][pools[r-1][combos[c][b]]][0];
				}
				if (countBits(ord) == r){
					// found subset, check for erasables
					var match = new Array();
					for (f=0; f<r; f++){
						match[f] = pools[r-1][combos[c][f]];
					}
					returnValue = getSubsetValuesInColumn(column, ord, match);
					if (returnValue !== null){
						return returnValue;
					}
				}
			}
		}
	}
	return null;
} // end findHiddenSubsetsInColumns()

function getSubsetValuesInColumn(column,ord,matches){
	var listOfNumbers = new ListOfCoordinates();
	var listOfErasableNumbers = new ListOfCoordinates();
	var returnValue = [listOfNumbers, listOfErasableNumbers];
	var tileCoords = new Array();
	var tmpOrd = ord;

	// get all of the coordinates for the blue numbers
	var match = new Array();
	for(var m=0; m<matches.length; m++){
		match[m] = columns[column][matches[m]][0];
	}
	for(var m=0; m<matches.length; m++){
		for(var b=0; b<dim; b++){
			if(match[m] %2 == 1){
				listOfNumbers.push(b, column, matches[m]);
			}
			match[m] = match[m] >>> 1;
		}
	}
	// get all of the coordinates for the yellow numbers
	// get coords for tiles
	for(var b=0; b < dim; b++){
		if(tmpOrd % 2 == 1){
			var next = tileCoords.length;
			tileCoords[next]= new Coordinate();
			tileCoords[next].row = b;
			tileCoords[next].column = column;
		}
		tmpOrd = tmpOrd >>> 1;
	}
	// check for other numbers in those tiles
	for (var v=0; v<dim; v++){
		var skip = false;
		for (m=0; m<matches.length; m++){
			if (v == matches[m]){
				skip = true;
				break;
			}
		}
		if (skip){
			continue;
		}
		for (m=0; m<tileCoords.length; m++){
			if (smallNumbers[tileCoords[m].row][tileCoords[m].column][v].data !== ""){
				listOfErasableNumbers.push(tileCoords[m].row, tileCoords[m].column, v);
			}
		}
	}
	
	if (listOfErasableNumbers.getSize() > 0){								
		return returnValue;
	}
	return null;
} // end getSubsetValuesInColumn()

function findXWingInRows(){
	var listOfNumbers = new ListOfCoordinates();
	var listOfErasableNumbers = new ListOfCoordinates();
	var returnValue = [listOfNumbers, listOfErasableNumbers];
	for (var number=0; number<dim; number++){
		// get pool of "twos"
		pools = new Array();
		for(var row=0; row<dim; row++){
			if(numbersR[number][row][1] == 2){
				pools[pools.length] = row;
			}
		}
		var pool = pools.length;
		if(pool >= 2){
			getCombos(pool, 2);
			for(var c=0; c<combos.length; c++){
				// compare for equal
				if (numbersR[number][pools[combos[c][0]]][0] == numbersR[number][pools[combos[c][1]]][0]){
					// found subset, check for erasables
					var match = new Array();
					match[0] = pools[combos[c][0]];
					match[1] = pools[combos[c][1]];
					returnValue = getXWingValuesInColumns(number, match);
					if (returnValue !== null){
						return returnValue;
					}
				}
			}
		}
	}
	return null;
}

function getXWingValuesInColumns(number, match){
	var listOfNumbers = new ListOfCoordinates();
	var listOfErasableNumbers = new ListOfCoordinates();
	var returnValue = [listOfNumbers, listOfErasableNumbers];
	var comp = numbersR[number][match[0]][0];
	// get coords for blue numbers
	for (var b=0; b<dim; b++){
		if (comp % 2 == 1){
			listOfNumbers.push(match[0], b, number);
			listOfNumbers.push(match[1], b, number);
		}
		comp = comp >>> 1;
	}
	// get all of the coordinates for the yellow numbers
	for (var row=0; row<dim; row++){
		if(row == match[0] || row == match[1]){
			continue;
		}
		var andd = numbersR[number][match[0]][0] & numbersR[number][row][0];
		if (andd > 0){
			for(var b=0; b<dim; b++){
				if(andd % 2 == 1){
					listOfErasableNumbers.push(row, b, number);
				}
				andd = andd >>> 1;
			}
		}
	}
	if(listOfErasableNumbers.getSize() > 0){
		return returnValue;
	}
	return null;
}

function findXWingInColumns(){
	var listOfNumbers = new ListOfCoordinates();
	var listOfErasableNumbers = new ListOfCoordinates();
	var returnValue = [listOfNumbers, listOfErasableNumbers];
	for (var number=0; number<dim; number++){
		// get pool of "twos"
		pools = new Array();
		for(var col=0; col<dim; col++){
			if(numbersC[number][col][1] == 2){
				pools[pools.length] = col;
			}
		}
		var pool = pools.length;
		if(pool >= 2){
			getCombos(pool, 2);
			for(var c=0; c<combos.length; c++){
				// compare for equal
				if (numbersC[number][pools[combos[c][0]]][0] == numbersC[number][pools[combos[c][1]]][0]){
					// found subset, check for erasables
					var match = new Array();
					match[0] = pools[combos[c][0]];
					match[1] = pools[combos[c][1]];
					returnValue = getXWingValuesInRows(number, match);
					if (returnValue !== null){
						return returnValue;
					}
				}
			}
		}
	}
	return null;
}

function getXWingValuesInRows(number, match){
	var listOfNumbers = new ListOfCoordinates();
	var listOfErasableNumbers = new ListOfCoordinates();
	var returnValue = [listOfNumbers, listOfErasableNumbers];
	var comp = numbersC[number][match[0]][0];
	// get coords for blue numbers
	for (var b=0; b<dim; b++){
		if (comp % 2 == 1){
			listOfNumbers.push(b, match[0], number);
			listOfNumbers.push(b, match[1], number);
		}
		comp = comp >>> 1;
	}
	// get all of the coordinates for the yellow numbers
	for (var col=0; col<dim; col++){
		if(col == match[0] || col == match[1]){
			continue;
		}
		var andd = numbersC[number][match[0]][0] & numbersC[number][col][0];
		if (andd > 0){
			for(var b=0; b<dim; b++){
				if(andd % 2 == 1){
					listOfErasableNumbers.push(b, col, number);
				}
				andd = andd >>> 1;
			}
		}
	}
	if(listOfErasableNumbers.getSize() > 0){
		return returnValue;
	}
	return null;
}

function findSwordfishInColumns(){
	var listOfNumbers = new ListOfCoordinates();
	var listOfErasableNumbers = new ListOfCoordinates();
	var returnValue = [listOfNumbers, listOfErasableNumbers];
	for (var number=0; number<dim; number++){
		// get pool of "twos" & "threes
		pools = new Array();
		for(var col=0; col<dim; col++){
			if(numbersC[number][col][1] == 2 || numbersC[number][col][1] == 3){
				pools[pools.length] = col;
			}
		}
		var pool = pools.length;
		if(pool >= 3){
			getCombos(pool, 3);
			for(var c=0; c<combos.length; c++){
				// or together, look for 3 bits
				var ord = 0;
				for(var b=0; b<3; b++){
					ord = ord | numbersC[number][pools[combos[c][b]]][0];
				}
				if (countBits(ord) == 3){
					// found subset, check for erasables
					var match = new Array();
					match[0] = pools[combos[c][0]];
					match[1] = pools[combos[c][1]];
					match[2] = pools[combos[c][2]];
					returnValue = getSwordfishValuesInRows(number, ord, match);
					if (returnValue !== null){
						return returnValue;
					}
				}
			}
		}
	}
	return null;
}

function getSwordfishValuesInRows(number, ord, matches){
	var listOfNumbers = new ListOfCoordinates();
	var listOfErasableNumbers = new ListOfCoordinates();
	var returnValue = [listOfNumbers, listOfErasableNumbers];
	var tileCoords = new Array();
	var tmpOrd = ord;
	
	// get all of the coordinates for the blue numbers
	var match = new Array();
	for(var m=0; m<matches.length; m++){
		match[m] = numbersC[number][matches[m]][0];
	}
	for(var m=0; m<match.length; m++){
		for(var b=0; b<dim; b++){
			if(match[m] %2 == 1){
				listOfNumbers.push(b, matches[m], number);
			}
		match[m] = match[m] >>> 1;
		}		
	}
	// get all of the coordinates for the yellow numbers
	for (var col=0; col<dim; col++){
		if(col == matches[0] || col == matches[1] || col == matches[2]){
			continue;
		}
		var andd = ord & numbersC[number][col][0];
		if (andd > 0){
			for(var b=0; b<dim; b++){
				if(andd % 2 == 1){
					listOfErasableNumbers.push(b, col, number);
				}
				andd = andd >>> 1;
			}
		}
	}
	if(listOfErasableNumbers.getSize() > 0){
		return returnValue;
	}
	return null;
}

function findSwordfishInRows(){
	var listOfNumbers = new ListOfCoordinates();
	var listOfErasableNumbers = new ListOfCoordinates();
	var returnValue = [listOfNumbers, listOfErasableNumbers];
	for (var number=0; number<dim; number++){
		// get pool of "twos" & "threes
		pools = new Array();
		for(var row=0; row<dim; row++){
			if(numbersR[number][row][1] == 2 || numbersR[number][row][1] == 3){
				pools[pools.length] = row;
			}
		}
		var pool = pools.length;
		if(pool >= 3){
			getCombos(pool, 3);
			for(var c=0; c<combos.length; c++){
				// or together, look for 3 bits
				var ord = 0;
				for(var b=0; b<3; b++){
					ord = ord | numbersR[number][pools[combos[c][b]]][0];
				}
				if (countBits(ord) == 3){
					// found subset, check for erasables
					var match = new Array();
					match[0] = pools[combos[c][0]];
					match[1] = pools[combos[c][1]];
					match[2] = pools[combos[c][2]];
					returnValue = getSwordfishValuesInColumns(number, ord, match);
					if (returnValue !== null){
						return returnValue;
					}
				}
			}
		}
	}
	return null;
}

function getSwordfishValuesInColumns(number, ord, matches){
	var listOfNumbers = new ListOfCoordinates();
	var listOfErasableNumbers = new ListOfCoordinates();
	var returnValue = [listOfNumbers, listOfErasableNumbers];
	var tileCoords = new Array();
	var tmpOrd = ord;
	
	// get all of the coordinates for the blue numbers
	var match = new Array();
	for(var m=0; m<matches.length; m++){
		match[m] = numbersR[number][matches[m]][0];
	}
	for(var m=0; m<match.length; m++){
		for(var b=0; b<dim; b++){
			if(match[m] %2 == 1){
				listOfNumbers.push(matches[m], b, number);
			}
		match[m] = match[m] >>> 1;
		}		
	}
	// get all of the coordinates for the yellow numbers
	for (var row=0; row<dim; row++){
		if(row == matches[0] || row == matches[1] || row == matches[2]){
			continue;
		}
		var andd = ord & numbersR[number][row][0];
		if (andd > 0){
			for(var b=0; b<dim; b++){
				if(andd % 2 == 1){
					listOfErasableNumbers.push(row, b, number);
				}
				andd = andd >>> 1;
			}
		}
	}
	if(listOfErasableNumbers.getSize() > 0){
		return returnValue;
	}
	return null;
}

function reverseHint(solve){
	var hintSolution = null;
	var blueNumbers, yellowNumbers;
	var coord;
	
	// finished
	hintSolution = checkForCompletedPuzzle();
	if (hintSolution !== null){
		showHintText(14,0);
		return;
	}

	// error
	hintSolution = findAnEmptyTile();
	if (hintSolution !== null){
		tintTile(hintSolution, COLOR_RED);
		coord = hintSolution.getCoord(1);
		showHintText(6, coord.number);
		return;
	}	

	// error
	hintSolution = findAMissingNumberInARow();
	if (hintSolution !== null){
		tintRow(hintSolution, COLOR_RED);
		coord = hintSolution.getCoord(1);
		showHintText(7, coord.number);
		return;
	}
	
	// error
	hintSolution = findAMissingNumberInAColumn();
	if (hintSolution !== null){
		tintColumn(hintSolution, COLOR_RED);
		coord = hintSolution.getCoord(1);
		showHintText(8, coord.number);
		return;
	}
	
	// error
	hintSolution = findAMissingNumberInASquare();
	if (hintSolution !== null){
		tintSquare(hintSolution, COLOR_RED);
		coord = hintSolution.getCoord(1);
		showHintText(9, coord.number);
		return;
	}
	
	// Swordfish
	hintSolution = findSwordfishInColumns();
	if (hintSolution !== null){
		blueNumbers = hintSolution[0];
		yellowNumbers = hintSolution[1];
		if(solve){
			if (solve_level <= SOLVE_SWORDFISH){
				reverseSolvePuzzle(blueNumbers, yellowNumbers);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(blueNumbers, COLOR_BLUE);
			tintSmallNumbers(yellowNumbers, COLOR_YELLOW);
			showHintText(17,0);
		}
		return;
	}

	// Swordfish
	hintSolution = findSwordfishInRows();
	if (hintSolution !== null){
		blueNumbers = hintSolution[0];
		yellowNumbers = hintSolution[1];
		if(solve){
			if (solve_level <= SOLVE_SWORDFISH){
				reverseSolvePuzzle(blueNumbers, yellowNumbers);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(blueNumbers, COLOR_BLUE);
			tintSmallNumbers(yellowNumbers, COLOR_YELLOW);
			showHintText(17,0);
		}
		return;
	}

	// X-Wing
	hintSolution = findXWingInRows();
	if (hintSolution !== null){
		blueNumbers = hintSolution[0];
		yellowNumbers = hintSolution[1];
		if(solve){
			if (solve_level <= SOLVE_XWING){
				reverseSolvePuzzle(blueNumbers, yellowNumbers);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(blueNumbers, COLOR_BLUE);
			tintSmallNumbers(yellowNumbers, COLOR_YELLOW);
			showHintText(16,0);
		}
		return;
	}
	
	// X-Wing
	hintSolution = findXWingInColumns();
	if (hintSolution !== null){
		blueNumbers = hintSolution[0];
		yellowNumbers = hintSolution[1];
		if(solve){
			if (solve_level <= SOLVE_XWING){
				reverseSolvePuzzle(blueNumbers, yellowNumbers);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(blueNumbers, COLOR_BLUE);
			tintSmallNumbers(yellowNumbers, COLOR_YELLOW);
			showHintText(16,0);
		}
		return;
	}
	
	// Hidden Subset
	hintSolution = findHiddenSubsetsInSquares();
	if (hintSolution !== null){
		blueNumbers = hintSolution[0];
		yellowNumbers = hintSolution[1];
		if(solve){
			if (solve_level <= SOLVE_HIDDEN_SUBSET){
				reverseSolvePuzzle(blueNumbers, yellowNumbers);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(blueNumbers, COLOR_BLUE);
			tintSmallNumbers(yellowNumbers, COLOR_YELLOW);
			showHintText(13,0);
		}
		return;
	}
	
	// Hidden Subset
	hintSolution = findHiddenSubsetsInRows();
	if (hintSolution !== null){
		blueNumbers = hintSolution[0];
		yellowNumbers = hintSolution[1];
		if(solve){
			if (solve_level <= SOLVE_HIDDEN_SUBSET){
				reverseSolvePuzzle(blueNumbers, yellowNumbers);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(blueNumbers, COLOR_BLUE);
			tintSmallNumbers(yellowNumbers, COLOR_YELLOW);
			showHintText(13,0);
		}
		return;
	}
	
	// Hidden Subset
	hintSolution = findHiddenSubsetsInColumns();
	if (hintSolution !== null){
		blueNumbers = hintSolution[0];
		yellowNumbers = hintSolution[1];
		if(solve){
			if (solve_level <= SOLVE_HIDDEN_SUBSET){
				reverseSolvePuzzle(blueNumbers, yellowNumbers);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(blueNumbers, COLOR_BLUE);
			tintSmallNumbers(yellowNumbers, COLOR_YELLOW);
			showHintText(13,0);
		}
		return;
	}
	
	// Naked Subset
	hintSolution = findNakedPairInASquare();
	if (hintSolution !== null){
		blueNumbers = hintSolution[0];
		yellowNumbers = hintSolution[1];
		if(solve){
			if (solve_level <= SOLVE_NAKED_SUBSET){
				reverseSolvePuzzle(blueNumbers, yellowNumbers);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(blueNumbers, COLOR_BLUE);
			tintSmallNumbers(yellowNumbers, COLOR_YELLOW);
			showHintText(12,0);
		}
		return;
	}
	
	// Naked Subset
	hintSolution = findNakedPairInARow();
	if (hintSolution !== null){
		blueNumbers = hintSolution[0];
		yellowNumbers = hintSolution[1];
		if(solve){
			if (solve_level <= SOLVE_NAKED_SUBSET){
				reverseSolvePuzzle(blueNumbers, yellowNumbers);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(blueNumbers, COLOR_BLUE);
			tintSmallNumbers(yellowNumbers, COLOR_YELLOW);
			showHintText(12,0);
		}
		return;
	}
	
	// Naked Subset
	hintSolution = findNakedPairInAColumn();
	if (hintSolution !== null){
		blueNumbers = hintSolution[0];
		yellowNumbers = hintSolution[1];
		if(solve){
			if (solve_level <= SOLVE_NAKED_SUBSET){
				reverseSolvePuzzle(blueNumbers, yellowNumbers);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(blueNumbers, COLOR_BLUE);
			tintSmallNumbers(yellowNumbers, COLOR_YELLOW);
			showHintText(12,0);
		}
		return;
	}
	
	// Block to Row/Col
	hintSolution = findBlockToRowOrCol();
	if (hintSolution !== null){
		blueNumbers = hintSolution[0];
		yellowNumbers = hintSolution[1];
		if(solve){
			if (solve_level <= SOLVE_BLOCK_RC){
				reverseSolvePuzzle(blueNumbers, yellowNumbers);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(blueNumbers, COLOR_BLUE);
			tintSmallNumbers(yellowNumbers, COLOR_YELLOW);
			showHintText(5,hintSolution[0].getCoord(1).number);
		}
		return;
	}
	
	// Block Block
	hintSolution = findRowToBlock();
	if (hintSolution !== null){
		blueNumbers = hintSolution[0];
		yellowNumbers = hintSolution[1];
		if(solve){
			if (solve_level <= SOLVE_BLOCK_BLOCK){
				reverseSolvePuzzle(blueNumbers, yellowNumbers);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(blueNumbers, COLOR_BLUE);
			tintSmallNumbers(yellowNumbers, COLOR_YELLOW);
			showHintText(10,hintSolution[0].getCoord(1).number);
		}
		return;
	}
	
	// Block Block
	hintSolution = findColumnToBlock();
	if (hintSolution !== null){
		blueNumbers = hintSolution[0];
		yellowNumbers = hintSolution[1];
		if(solve){
			if (solve_level <= SOLVE_BLOCK_BLOCK){
				reverseSolvePuzzle(blueNumbers, yellowNumbers);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(blueNumbers, COLOR_BLUE);
			tintSmallNumbers(yellowNumbers, COLOR_YELLOW);
			showHintText(11,hintSolution[0].getCoord(1).number);
		}
		return;
	}
	
	// Hidden Single
	hintSolution = findHiddenSingleInASquare();
	if (hintSolution !== null){
		if(solve){
			if (solve_level <= SOLVE_HIDDEN_SINGLE){
				reverseSolvePuzzle(hintSolution, null);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(hintSolution, COLOR_RED);
			coord = hintSolution.getCoord(1);
			showHintText(2, coord.number);
		}
		return;
	}
	
	// Hidden Single
	hintSolution = findHiddenSingleInARow();
	if (hintSolution !== null){
		if(solve){
			if (solve_level <= SOLVE_HIDDEN_SINGLE){
				reverseSolvePuzzle(hintSolution, null);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(hintSolution, COLOR_RED);
			coord = hintSolution.getCoord(1);
			showHintText(3, coord.number);
		}
		return;
	}
	
	// Hidden Single
	hintSolution = findHiddenSingleInAColumn();
	if (hintSolution !== null){
		if(solve){
			if (solve_level <= SOLVE_HIDDEN_SINGLE){
				reverseSolvePuzzle(hintSolution, null);
			} else {
				showHintText(15, 0);
				return;
			}
		} else {
			tintSmallNumbers(hintSolution, COLOR_RED);
			coord = hintSolution.getCoord(1);
			showHintText(4, coord.number);
		}
		return;
	}
	
	// Naked Single
	hintSolution = findNakedSingle();
	if (hintSolution !== null){
		if(solve && solve_level <= SOLVE_NAKED_SINGLE){
			reverseSolvePuzzle(hintSolution, null);
		} else {
			tintSmallNumbers(hintSolution, COLOR_RED);
			coord = hintSolution.getCoord(1);
			showHintText(1, coord.number);
		}
		return;
	}
	
	// None found
	showHintText(0);
}// end reverseHint()

function countBits(number){
	var returnValue = 0;
	for(var f=0; f<dim; f++){
		returnValue += number%2;
		number = number >>> 1;
	}
	return returnValue;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
*	
*														File 
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
function inputFromClipboard(){
	var clip = system.clipboard;
	inputPuzzle(clip);
	logPuzzle(clip);
}

function outputToClipboard(){
	system.clipboard = captureToString();
}

function saveToShortFile(){
	var outputString = captureToString();
	outputToFile(outputString, ".sdk");
}

function captureToString(){
	var outputString = "";
	for (var i=0; i<dim; i++){
		for (var j=0; j<dim; j++){
			if (bigNumbers[i][j].data !== "") {
				outputString += default_candidates.charAt(candidates.indexOf(bigNumbers[i][j].data));
			} else {
				outputString += ".";
			}
		}
	}
	return outputString;
}

function saveToFile(){
	var puzzleArray = puzzleToArray();
	outputToFile(puzzleArray, ".sdk");
} // end saveToFile()

function puzzleToArray(){
	var fileArray = new Array();
	fileArray[0]= "Sudoku Solver File";
	fileArray[1]= widget.version;
	fileArray[2]= dim;

	// get small number coordinates
	for (var i=0; i<dim; i++){
		for (var j=0; j<dim; j++){
			for (var k=0; k<dim; k++){
				if (smallNumbers[i][j][k].data !== ""){
					fileArray[fileArray.length] = i;
					fileArray[fileArray.length] = j;
					fileArray[fileArray.length] = default_candidates.charAt(candidates.indexOf(smallNumbers[i][j][k].data));
				}
			}
		}
	}
	// get big number coordinates
	fileArray[fileArray.length] = "BIG";
	fileArray[fileArray.length] = "BIG";
	fileArray[fileArray.length] = "BIG";
	for (var i=0; i<dim; i++){
		for (var j=0; j<dim; j++){
			if (bigNumbers[i][j].data !== "") {
				fileArray[fileArray.length] = i;
				fileArray[fileArray.length] = j;
				fileArray[fileArray.length] = default_candidates.charAt(candidates.indexOf(bigNumbers[i][j].data));
			}
		}
	}
	// get undoList
	fileArray[fileArray.length] = "UNDO";
	fileArray[fileArray.length] = "UNDO";
	fileArray[fileArray.length] = "UNDO";
	var undoListSize = undoList.length;
	for (var x=0; x<undoListSize; x++){
		var undoSet = undoList[x];
		var undoSize = undoSet.size;
		for(var z=0; z<undoSize; z++){
			fileArray[fileArray.length] = undoSet.row[z];
			fileArray[fileArray.length] = undoSet.column[z];
			fileArray[fileArray.length] = default_candidates[undoSet.number[z]];
		}
		fileArray[fileArray.length] = ".";
		fileArray[fileArray.length] = ".";
		fileArray[fileArray.length] = ".";
	}
	return fileArray;
}

function outputToFile(output, ext){
	var outPutFile = saveAs(ext);
	if (outPutFile !== null){
		filesystem.writeFile(outPutFile, output);
	}
}// end outputToFile)

function paste(){
	var formfields = Array();
	var formResults = Array();
	var success = true;
	formfields[0] = new FormField();
	formfields[0].name = 'name1';
	formfields[0].type = 'text';
	formfields[0].title = '';
	formfields[0].defaultValue = preferences.paste_text.value;
	formfields[0].description = 'Paste or type a puzzle in the format of "..3.6..4. and so on"';
	formResults = form(formfields, 'Input a Puzzle', 'Input');
	if (formResults !== null){
		preferences.paste_text.value = formResults[0];
		success = inputPuzzle(formResults[0]);
	}
	if (!success){
		paste();
	} else {
		logPuzzle(formResults[0]);
	}
}

function readFromShortFile(fileArray){
	// put all the data into one string
	var fileList = "";
	var numberList = "";
	for (var x=0; x < fileArray.length; x++){
		if(fileArray[x] == " " || fileArray[x] == "\n"){
			continue;
		}
		fileList += fileArray[x];
	}
	// remove spaces and special characters
	for (var x=0; x<fileList.length; x++){
		var character = fileList.charAt(x);
		if(character != " " && character != "-" && character != "+" && character != "|"){
			numberList += character;
		}
	}
	inputPuzzle(numberList);
	logPuzzle(numberList);
} // end readFromShortFile()

function inputPuzzle(numberList){
	// check for data size (allow 9x9 or 12x12)
	if ((numberList.length != 81) && (numberList.length != 144) && (numberList.length != 256) ){
		alert("Not a valid Sudoku Solver file. Length must be 9x9 or 12x12 or 16x16.");
		return false;
	}
	
	// clear all numbers, set dimensions
	dim = Math.sqrt(numberList.length);
	boardSize(dim);
	
	// fill the big numbers
	var i,j;
	for (var x=0; x < numberList.length; x++){
		i = Math.floor(x/dim);
		j = x%dim;
		var chosenCharacter = numberList.charAt(x);
		if ((chosenCharacter < "1") || (chosenCharacter > "9")){
			if ((chosenCharacter < "A") || (chosenCharacter > "G")){
				continue;
			}
		}
		var numberIndex = default_candidates.indexOf(chosenCharacter);
		if (numberIndex < 0){
			alert("Not a valid Sudoku Solver file. Contains the unexpected char: " + chosenCharacter);
			return false;
		}
		smallNumberHandler(i, j, numberIndex);
	}
	
	// clear the undo list
	clearUndo();
	return true;
} // end inputPuzzle()

function clearUndo(){
	undoList.length = 0;
	undo_text.opacity = 100;
}

function readFromFile(){
	var inputFile = chooseFile(".sdk");
	if (inputFile !== null){
		readFile(inputFile);
	}
}

function readFile(inputFile){
	fileArray = filesystem.readFile(inputFile, true);
	if(fileArray === null){
		return;
	}
	if (fileArray[0] != "Sudoku Solver File"){
		readFromShortFile(fileArray);
		return;
	} else {
		var fileVersion = fileArray[1];
		var fileDimensions = fileArray[2];
		var fileSize = fileArray.length;
		// clear all numbers and undoList
		dim = parseInt(fileDimensions,10);
		boardSize(dim);
		for (var i=0; i<dim; i++){
			for (var j=0; j<dim; j++){
				if (preferences.colorOn.value > 0){
					tiles[i][j].colorize = board_color;
				} else {
					tiles[i][j].colorize = "";
				}
				bigNumbers[i][j].data = "";
				for (var k=0; k<dim; k++){
					smallNumbers[i][j][k].data = "";
					smallNumbers[i][j][k].color = COLOR_NUMBERS;
				}
			}
		}
		undoList.length = 0;
		// clear data
		eraseData();
	
		// fill the numbers and undoList			
		var group = "SMALL";
		var undo = new undoObject();
		for(var f=3; f<fileSize; f=f+3){
			var i = fileArray[f];
			var j = fileArray[f+1];
			var datum = fileArray[f+2];
			var k = default_candidates.indexOf(datum);
			if(i=="BIG" || i=="UNDO"){
				group = i;
				continue;
			}
			if (group == "SMALL"){
				// fill small numbers
				var square = tileToSquare[i][j];
				var posit = squareToTileJ[i][j];
				smallNumbers[i][j][k].data = candidates.charAt(k);
				squares[square][k][0] = squares[square][k][0] | (1 << posit);
				squares[square][k][1]++;
				rows[i][k][0] = rows[i][k][0] | (1 << j);
				rows[i][k][1]++;
				columns[j][k][0] = columns[j][k][0] | (1 << i);
				columns[j][k][1]++;
				numbersR[k][i][0] = numbersR[k][i][0] | (1 << j);
				numbersR[k][i][1]++;
				numbersC[k][j][0] = numbersC[k][j][0] | (1 << i);
				numbersC[k][j][1]++;
				continue;
			}
			if (group == "BIG"){
				// fill big numbers
				bigNumbers[i][j].data = candidates.charAt(k);
				continue;
			}
			if (group == "UNDO"){
				// fill undoList
				undo_text.opacity = 255;
				if (i == "."){
					undoList[undoList.length]=undo;
					undo = new undoObject();
				}else{
					undo.push(i,j,k);
				}
			}
		}
		logPuzzle(captureToString());
	}
}// end readFile()

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
*	
*														Generate Puzzle
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*
Index=0		Single candidate                             1
Index=1		Unique candidate                             1
Index=2		Pairs/triplets                               3
Index=3		Block/column/row interaction                 2
Index=4		Block/block interaction                      2
Index=5		Hidden pairs                                 3
Index=6		Hidden triplets                              3
Index=7		X-wings                                      3
Index=8		Guessing                                     4
*/
var puzzle;
function generateGame(){
	puzzle = runCommand("./sudgen.exe " + dim + " " + (parseInt(difficulty,10)+1)+ " 1 1 3 2 2 3 3 3 4");
	if (puzzle.length == 16*16 || puzzle.length == 9*9){
		inputPuzzle(puzzle);
		logPuzzle(puzzle);
		text1.data = "";
	} else {
		text1.data = "Sorry. Something inexplicable has prevented puzzle generation.";
		text2.data = "If I were you, I would complain about it.";
		setUntintButton();
	}
}

function logPuzzle(puzzle){
	if(recent_puzzles_array.length > 0){
		if (puzzle == recent_puzzles_array[0]){
			return;
		}
	}
	recent_puzzles_array.unshift(puzzle);
	while (recent_puzzles_array.length > recent_puzzles_array_limit){
		recent_puzzles_array.pop();
	}
}

function loadRecentPuzzle(){
	var formfields = Array();
	formfields[0] = new FormField();
	formfields[0].name = 'recent_puzzles';
	formfields[0].type = 'popup';
	formfields[0].title = 'Recent Puzzles';
	formfields[0].option = recent_puzzles_array;
	formfields[0].optionValue = new Array();
	for (var i=0; i< formfields[0].option.length; i++){
		formfields[0].optionValue[i] = i;
	}
	formfields[1] = new FormField();
	formfields[1].name = 'clear_list';
	formfields[1].type = 'checkbox';
	formfields[1].title = 'Clear Recent Puzzles List';
	
	formResults = form(formfields, 'Open Recent Puzzle', 'OK');
	if (formResults != null) {
		var puzzle_index = parseInt(formResults[0], 10);
		var puzzle = recent_puzzles_array[puzzle_index];
		var clear_puzzle_list = formResults[1];
		if (clear_puzzle_list === '1'){
			clearRecent();
			return;
		}
		if (puzzle.length == 16*16 || puzzle.length == 9*9){
			inputPuzzle(puzzle);
			recent_puzzles_array.unshift(recent_puzzles_array.splice(puzzle_index,1));
		}
	}
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
*	
*														Actions
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
function minimizeView(){
	main_window.visible = false;
	markup_window.visible = false;
	minimize_window.visible = true;
}

function showMainWindow(){
	minimize_window.visible = false;
	main_window.visible = true;
}

function setInitialPosition(){
	minimize_window.hOffset = main_window.hOffset + minimize_button.hOffset;
	minimize_window.vOffset = main_window.vOffset + minimize_button.vOffset;
}

function savePuzzle(){
	var saveFileName = system.widgetDataFolder + "/temp.sdk";
	var puzzle = puzzleToArray();
	filesystem.writeFile(saveFileName, puzzle);
}

function restorePuzzle(){
	var saveFileName = system.widgetDataFolder + "/temp.sdk";
	readFile(saveFileName);
}

function saveRecent(){
	var save_recent_name = system.widgetDataFolder + "/recent.txt";
	filesystem.writeFile(save_recent_name, recent_puzzles_array);
}

function restoreRecent(){
	var save_recent_name = system.widgetDataFolder + "/recent.txt";
	recent_puzzles_array = filesystem.readFile(save_recent_name, true);
	if(recent_puzzles_array === null){
		recent_puzzles_array = new Array();
	}
}

function clearRecent(){
	//recent_puzzles_array = new Array();
	recent_puzzles_array.splice(1);
	saveRecent();
}

function dumpDataToFile(){
	var fileArray = new Array();
	fileArray[fileArray.length] = "";
	fileArray = fileArray.concat(dataToPrintArray(squares, "squares"));
	fileArray = fileArray.concat(dataToPrintArray(rows, "rows"));
	fileArray = fileArray.concat(dataToPrintArray(columns, "columns"));
	fileArray = fileArray.concat(dataToPrintArray(numbersR, "numbersR"));
	fileArray = fileArray.concat(dataToPrintArray(numbersC, "numbersC"));
	outputToFile(fileArray, ".dta");
}

function dataToPrintArray(data_array, title){
	var returnValue = new Array();
	for (var x=0; x<data_array.length; x++){
		returnValue[returnValue.length] = title + "(" + (parseInt(x,10)+1) + ")";
		for(var k=0; k<dim; k++){
			var binary = data_array[x][k][0].toString(2);
			while (binary.length < dim){
				binary = "0" + binary;
			}
			returnValue[returnValue.length] = (parseInt(k,10)+1 + ": " + binary + "  " + data_array[x][k][1]);
		}
		returnValue[returnValue.length] = "";
	}
	return returnValue;
}

function dropPuzzle(){
	var dropped = system.event.data;
	var split_file;
	if(dropped[0] == "filenames"){
		readFile(dropped[1]);
	}
}

function newMouseDown(){
	new_game_button.src="Resources/" + tile_texture + "/pressed_button.jpg";
	new_game_text.vOffset = new_game_text.vOffset + 2;
	new_game_text.hOffset = new_game_text.hOffset + 2;
	text1.data = "Generating new puzzle. Please be patient.";
	text2.data = "";
}

function newMouseUp(){
	new_game_button.src="Resources/" + tile_texture + "/button.jpg";
	new_game_text.vOffset = new_game_text.vOffset - 2;
	new_game_text.hOffset = new_game_text.hOffset - 2;
	if(!no_questions){
		if (checkForCompletedPuzzle() === null && checkForUnstartedPuzzle() === null){
			var response = alert("You're not done yet. Are you sure you want to start a New puzzle?", "Yes, gimme a new puzzle." , "No, wait, I want to finish this." , "Yes, doggonit, and don't ask me stupid questions");
			switch(response){
				case 1:
					break;
				case 2:
					return;
					break;
				case 3:
					preferences.no_questions.value = 1;
					no_questions = true;
					break;
				default:
					return;
			}
		}
	}
	generateGame();
}

function undoMouseDown(){
	undo_button.src="Resources/" + tile_texture + "/pressed_button.jpg";
	undo_text.vOffset = undo_text.vOffset + 2;
	undo_text.hOffset = undo_text.hOffset + 2;
}

function undoMouseUp(){
	undo_button.src="Resources/" + tile_texture + "/button.jpg";
	undo_text.vOffset = undo_text.vOffset - 2;
	undo_text.hOffset = undo_text.hOffset - 2;
	var mod = system.event.modifiers;
	if(mod != "option"){
		undo();
	} else {
		undoAll();
	}
}

function hintMouseDown(){
	hint_button.src="Resources/" + tile_texture + "/pressed_button.jpg";
	hint_text.vOffset = hint_text.vOffset + 2;
	hint_text.hOffset = hint_text.hOffset + 2;
}

function hintMouseUp(){
	hint_button.src="Resources/" + tile_texture + "/button.jpg";
	hint_text.vOffset = hint_text.vOffset - 2;
	hint_text.hOffset = hint_text.hOffset - 2;
	if(no_help){
		text1.data = "No hint for you!";
		text2.data = "You choose the Pltboi Option, you live by the Pltboi Rules.";
		setUntintButton();
		return;
	}
	var mod = system.event.modifiers;
	switch (mod){
		case 'option':
			showHint(true);
		break;
		case 'shift':
			reverseHint(false);
		break;
		case 'shift+option':
			reverseHint(true);
		break;
		default:
			showHint(false);
	}
}

function closeWindow(){
	markup_window.visible = false;
}

function tileMouseDown(i, j){
	var x = system.event.screenX;
	var y = system.event.screenY;
	markup_window.hOffset = x-105;
	markup_window.vOffset = y;
	markup_window.tileI = i;
	markup_window.tileJ = j;
	markup_window.visible = true;
	markup_window.level = "topmost";
}

function markupBigNumber(number){
	var i = markup_window.tileI;
	var j = markup_window.tileJ;
	smallNumbers[i][j][number].data = candidates.charAt(number);
	smallNumberClick(i, j, number);
	markup_window.visible = false;
}

function markupSmallNumber(number){
	var i = markup_window.tileI;
	var j = markup_window.tileJ;
	smallNumbers[i][j][number].data = candidates.charAt(number);
	markup_window.visible = false;
}

function checkForUpdates(version){
	var verChk = new VersionChecker( "38124", "Sudoku Solver", version );
	verChk.checkVersion();
}

function parseKeyPress(){
	var V_KEY = "0x16";
	var C_KEY = "0x3";
	var Z_KEY = "0x1a";
	var key_string = system.event.keyString;
	var mod = system.event.modifiers;
	var key = system.event.key;
	if (key_string == V_KEY && mod == "control"){
		inputFromClipboard();
		return;
	}
	if (key_string == C_KEY && mod == "control"){
		outputToClipboard();
		return;
	}
	if (key_string == Z_KEY && mod == "shift+control"){
		undo();
		return;
	}
}

function smallNumbersMouseEnter(row, col, number){
	if (!superhint){
		return;
	}
	if(smallNumbers[row][col][number].data === ""){
		return;
	}
	showAllNumbers(row, col, number);
}

function smallNumbersMouseExit(number){
	if (!superhint){
		return;
	}
	restoreAllNumbers(number);
}

function showAllNumbers(row, col, number){
	var this_square = tileToSquare[row][col];
	
	for(var i=0; i < dim; i++){
		for(var j=0; j < dim; j++){
			if (i == row || j == col || this_square == tileToSquare[i][j]){
				if (smallNumbers[i][j][number].color == COLOR_NUMBERS){
					smallNumbers[i][j][number].color = COLOR_PINK;
				}
			}
		}
	}
}

function colorAllNumbers(row, col, number){
	var newColor = COLOR_RED;
	var colorIndex = RAINBOW.indexOf(smallNumbers[row][col][number].color);
	if (colorIndex >= 0){
		if (colorIndex > RAINBOW.length - 14){
			newColor = RAINBOW.substring(0, 7);
		} else {
			newColor = RAINBOW.substring(colorIndex+7, colorIndex+14);
		}
	}
	for(var i=0; i < dim; i++){
		for(var j=0; j < dim; j++){
			if (smallNumbers[i][j][number].data !== ""){
				smallNumbers[i][j][number].color = newColor;
			}
		}
	}
}

function restoreAllNumbers(number){
	for(var i=0; i < dim; i++){
		for(var j=0; j < dim; j++){
			for(var k=0; k < dim; k++){
				if (smallNumbers[i][j][k].color == COLOR_PINK){
					smallNumbers[i][j][k].color = COLOR_NUMBERS;
				}
			}
		}
	}
}

function bigNumberClick(i, j){
	bigNumbers[i][j].data = "";
}

function smallNumberClick(a, b, c){
	if(smallNumbers[a][b][c].data === ""){
		if(no_help){
			tileMouseDown(a,b);
		}
		return;
	}
	hideHintText();
	switch(system.event.modifiers){
		case "option":
			setUntintButton();
			var colorIndex = RAINBOW.indexOf(smallNumbers[a][b][c].color);
			if (colorIndex >= 0){
				if (colorIndex > RAINBOW.length - 14){
					smallNumbers[a][b][c].color = RAINBOW.substring(0, 7);
				} else {
					smallNumbers[a][b][c].color = RAINBOW.substring(colorIndex+7, colorIndex+14);
				}
			}
			break;
		case "shift":
			smallNumberHandler(a, b, c, true);
			break;
		case "shift+option":
			colorAllNumbers(a, b, c);
			setUntintButton();
			break;
		default:
			smallNumberHandler(a, b, c, false);
	}
}

function smallNumberHandler(a, b, c, shift){
	var undo = new undoObject();
	var square = tileToSquare[a][b];
	var posit = squareToTileJ[a][b];
 	if (shift){ // erase small number
 		smallNumbers[a][b][c].data = "";
		squares[square][c][0] = squares[square][c][0] & ~(1 << posit);
		squares[square][c][1]--;
		if(squares[square][c][1] < 0){
			squares[square][c][1] = 0;
		}
		rows[a][c][0] = rows[a][c][0] & ~(1 << b);
		rows[a][c][1]--;
		if(rows[a][c][1] < 0){
			rows[a][c][1] = 0;
		}

		columns[b][c][0] = columns[b][c][0] & ~(1 << a);
		columns[b][c][1]--;
		if(columns[b][c][1] < 0){
			columns[b][c][1] = 0;
		}
		
		numbersR[c][a][0] = numbersR[c][a][0] & ~(1 << b);
		numbersR[c][a][1]--;
		if(numbersR[c][a][1] < 0){
			numbersR[c][a][1] = 0;
		}

		numbersC[c][b][0] = numbersC[c][b][0] & ~(1 << a);
		numbersC[c][b][1]--;
		if(numbersC[c][b][1] < 0){
			numbersC[c][b][1] = 0;
		}

 		undo.push(a,b,c);
 	} else { // select small number
		for(var k=0; k < dim; k++){
			if(smallNumbers[a][b][k].data !== ""){
				smallNumbers[a][b][k].data = "";
				undo.push(a,b,k);
				squares[square][k][0] = squares[square][k][0] & ~(1 << posit);
				squares[square][k][1]--;
				if(squares[square][k][1] < 0){
					squares[square][k][1] = 0;
				}
				rows[a][k][0] = rows[a][k][0] & ~(1 << b);
				rows[a][k][1]--;
				if(rows[a][k][1] < 0){
					rows[a][k][1] = 0;
				}
				columns[b][k][0] = columns[b][k][0] & ~(1 << a);
				columns[b][k][1]--;
				if(columns[b][k][1] < 0){
					columns[b][k][1] = 0;
				}
				numbersR[k][a][0] = numbersR[k][a][0] & ~(1 << b);
				numbersR[k][a][1]--;
				if(numbersR[k][a][1] < 0){
					numbersR[k][a][1] = 0;
				}
		
				numbersC[k][b][0] = numbersC[k][b][0] & ~(1 << a);
				numbersC[k][b][1]--;
				if(numbersC[k][b][1] < 0){
					numbersC[k][b][1] = 0;
				}
			}
		}
		bigNumbers[a][b].data = candidates.charAt(c);
		if (!no_help){
			eraseNumbersInSquare(square,c,undo);
			eraseNumbersInRow(a,c,undo);
			eraseNumbersInColumn(b,c,undo);
		}
		if (testCompletedPuzzle()){
			puzzleCorrect();
		}
	}
	
	undoList[undoList.length] = undo;
	if(undoList.length > 0){	
		undo_text.opacity=255;
	}else{
		undo_text.opacity=100;
	}

	if(tool_tips){
		undo_button.tooltip = "Click to undo your last step.";
	}
}
		
function eraseNumbersInSquare(s,c,undo){
	for(var z=0; z<dim; z++){
		var i = squareToTileI[s][z];
		var j = squareToTileJ[s][z];
		var posit = squareToTileJ[i][j];
		var k = c;
		if(smallNumbers[i][j][k].data !== ""){
			smallNumbers[i][j][k].data = "";
			squares[s][k][0] = squares[s][k][0] & ~(1 << posit);
			squares[s][k][1] = squares[s][k][1] - 1;
			if(squares[s][k][1] < 0){
				squares[s][k][1] = 0;
			}
			rows[i][k][0] = rows[i][k][0] & ~(1 << j);
			rows[i][k][1]--;
			if(rows[i][k][1] < 0){
				rows[i][k][1] = 0;
			}
			columns[j][k][0] = columns[j][k][0] & ~(1 << i);
			columns[j][k][1]--;
			if(columns[j][k][1] < 0){
				columns[j][k][1] = 0;
			}
			numbersR[k][i][0] = numbersR[k][i][0] & ~(1 << j);
			numbersR[k][i][1]--;
			if(numbersR[k][i][1] < 0){
				numbersR[k][i][1] = 0;
			}
			numbersC[k][j][0] = numbersC[k][j][0] & ~(1 << i);
			numbersC[k][j][1]--;
			if(numbersC[k][j][1] < 0){
				numbersC[k][j][1] = 0;
			}

			undo.push(i,j,k);
		}
	}	
}// end eraseNumbersInSquare()

function eraseNumbersInRow(i,k,undo){
	for(var j=0; j<dim; j++){
		var posit = squareToTileJ[i][j];
		var s = tileToSquare[i][j];
		if(smallNumbers[i][j][k].data !== ""){
			smallNumbers[i][j][k].data = "";
			squares[s][k][0] = squares[s][k][0] & ~(1 << posit);
			squares[s][k][1] = squares[s][k][1] - 1;
			if(squares[s][k][1] < 0){
				squares[s][k][1] = 0;
			}
			rows[i][k][0] = rows[i][k][0] & ~(1 << j);
			rows[i][k][1]--;
			if(rows[i][k][1] < 0){
				rows[i][k][1] = 0;
			}
			columns[j][k][0] = columns[j][k][0] & ~(1 << i);
			columns[j][k][1]--;
			if(columns[j][k][1] < 0){
				columns[j][k][1] = 0;
			}
			numbersR[k][i][0] = numbersR[k][i][0] & ~(1 << j);
			numbersR[k][i][1]--;
			if(numbersR[k][i][1] < 0){
				numbersR[k][i][1] = 0;
			}
			numbersC[k][j][0] = numbersC[k][j][0] & ~(1 << i);
			numbersC[k][j][1]--;
			if(numbersC[k][j][1] < 0){
				numbersC[k][j][1] = 0;
			}

			undo.push(i,j,k);
		}
	}
}// end eraseNumbersInRow()

function eraseNumbersInColumn(j,k,undo){
	for(var i=0; i<dim; i++){
		var posit = squareToTileJ[i][j];
		var s = tileToSquare[i][j];
		if(smallNumbers[i][j][k].data !== ""){
			smallNumbers[i][j][k].data = "";
			squares[s][k][0] = squares[s][k][0] & ~(1 << posit);
			squares[s][k][1] = squares[s][k][1] - 1;
			if(squares[s][k][1] < 0){
				squares[s][k][1] = 0;
			}
			rows[i][k][0] = rows[i][k][0] & ~(1 << j);
			rows[i][k][1]--;
			if(rows[i][k][1] < 0){
				rows[i][k][1] = 0;
			}
			columns[j][k][0] = columns[j][k][0] & ~(1 << i);
			columns[j][k][1]--;
			if(columns[j][k][1] < 0){
				columns[j][k][1] = 0;
			}
			numbersR[k][i][0] = numbersR[k][i][0] & ~(1 << j);
			numbersR[k][i][1]--;
			if(numbersR[k][i][1] < 0){
				numbersR[k][i][1] = 0;
			}
			numbersC[k][j][0] = numbersC[k][j][0] & ~(1 << i);
			numbersC[k][j][1]--;
			if(numbersC[k][j][1] < 0){
				numbersC[k][j][1] = 0;
			}

			undo.push(i,j,k);
		}
	}
}// end eraseNumbersInColumn()

function difficultyMouseDown(){
	if (dim == 12){
		return;
	}
	difficulty_button.src = "Resources/" + tile_texture + "/pressed_button.jpg";
	difficulty_text.hOffset = difficulty_text.hOffset + 2;
	difficulty_text.vOffset = difficulty_text.vOffset + 2;
}

function difficultyMouseUp(){
	if (dim == 12){
		return;
	}
	difficulty = difficulty + 1;
	if (difficulty == difficulty_name.length - 1){
		difficulty = 0;
	}
	difficulty_button.src = "Resources/" + tile_texture + "/button.jpg";
	difficulty_text.hOffset = difficulty_text.hOffset - 2;
	difficulty_text.vOffset = difficulty_text.vOffset - 2;
	difficulty_text.data = difficulty_abbr[difficulty];
	preferences.difficulty.value = difficulty;
}

function checkSolveLevel(level, condition){
	switch (level){
		case SOLVE_NAKED_SINGLE:
			solveLevelNakedSingleMenuItem.checked = condition;
			break;
		case SOLVE_HIDDEN_SINGLE:
			solveLevelHiddenSingleMenuItem.checked = condition;
			break;
		case SOLVE_NAKED_SUBSET:
			solveLevelNakedSubsetMenuItem.checked = condition;
			break;
		case SOLVE_HIDDEN_SUBSET:
			solveLevelHiddenSubsetMenuItem.checked = condition;
			break;
		case SOLVE_BLOCK_RC:
			solveLevelBlockRCMenuItem.checked = condition;
			break;
		case SOLVE_BLOCK_BLOCK:
			solveLevelBlockBlockMenuItem.checked = condition;
			break;
		case SOLVE_XWING:
			solveLevelXWingMenuItem.checked = condition;
			break;
		case SOLVE_SWORDFISH:
			solveLevelSwordfishMenuItem.checked = condition;
			break;
		case SOLVE_ENTIRE_PUZZLE:
			solveLevelEntirePuzzleMenuItem.checked = condition;
			break;
	}
}

function changeSolveLevel(level){
	checkSolveLevel(solve_level, false);
	checkSolveLevel(level, true);
	solve_level = level;
	preferences.solve_level.value = level;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
*	
*														Initial Setup
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

function smallNumberHorizOffset(i,j,k){
	var dim_factor = Math.floor(Math.sqrt(dim));  // 9=3, 12=3, 16=4
	var dim_factor2 = Math.ceil(Math.sqrt(dim));  // 9=3, 12=4, 16=4
	var tile_offset = tiles[i][j].hOffset;
	var tile_width = tiles[i][j].width;
	var margin = Math.floor(tile_width/(dim_factor2+1));
	var printable_area = tile_width - 2 * margin;
	return (tiles[i][j].hOffset + margin +Math.floor((k % dim_factor)*(printable_area/(dim_factor-1))));
} 

function smallNumberVertOffset(i,j,k){
	var dim_factor = Math.ceil(Math.sqrt(dim));  // 9=3, 12=4, 16=4
	var dim_factor2 = Math.floor(Math.sqrt(dim));  // 9=3, 12=3, 16=4
	var tile_offset = tiles[i][j].vOffset;
	var tile_height = tiles[i][j].height;
	var margin = Math.floor(tile_height/((dim_factor+2)));
	var printable_area = tile_height - 2 * margin;
	return (tiles[i][j].vOffset + margin + Math.floor(Math.floor(k / dim_factor2)*(printable_area/(dim_factor-1))) + 4);

} 

function tileHorizOffset(j){
	var dim_factor = Math.ceil(Math.sqrt(dim));  // 9=3, 12=4, 16=4
	return (j*tile_size[size] + 15) + 2*(Math.floor(j/(dim_factor))); 
}

function tileVertOffset(i){
	var dim_factor = Math.floor(Math.sqrt(dim));  // 9=3, 12=3, 16=4
	return (i*tile_size[size] + 15) + 2*(Math.floor(i/dim_factor)); 
}

function resizeBackground(){
	var space_for_outline = 2*(Math.ceil(Math.sqrt(dim))+1);
	var space_for_border_edge = 13;
	var space_for_buttons = 100;
	var tile_height = tiles[0][0].height;
	var tile_width = tiles[0][0].width;
	
	frame_background.src = "Resources/" + tile_texture + "/frame_bg.jpg";
	frame_upper_left.src = "Resources/" + tile_texture + "/frame_corner.jpg";
	frame_upper_right.src = "Resources/" + tile_texture + "/frame_corner.jpg";
	frame_top.src = "Resources/" + tile_texture + "/frame_top.jpg";
	frame_bottom.src = "Resources/" + tile_texture + "/frame_top.jpg";
	frame_lower_left.src = "Resources/" + tile_texture + "/frame_corner.jpg";
	frame_lower_right.src = "Resources/" + tile_texture + "/frame_corner.jpg";
	frame_left.src = "Resources/" + tile_texture + "/frame_side.jpg";
	frame_right.src = "Resources/" + tile_texture + "/frame_side.jpg";
	
	background.width = dim*tile_width + space_for_outline ;
	background.height = dim*tile_height + space_for_outline;
	frame_background.width = background.width;
	frame_background.vOffset = background.vOffset + background.height;
	frame_top.width = background.width;
	frame_upper_right.hOffset = frame_top.hOffset + frame_top.width;
	frame_left.height = background.height + frame_background.height;
	frame_right.height = frame_left.height;
	frame_right.hOffset = background.hOffset + background.width;
	frame_bottom.width = background.width;
	frame_bottom.vOffset = background.vOffset + background.height + frame_background.height;
	frame_lower_left.vOffset = frame_bottom.vOffset;
	frame_lower_right.vOffset = frame_bottom.vOffset;
	frame_lower_right.hOffset = frame_right.hOffset;
	
	main_window.width = background.width + 26;
	main_window.height = background.height + frame_background.height + 26;
}

function placeButtons(){
	var vertical_edge = background.height;
	var left_edge = background.hOffset;
	var right_edge = left_edge + background.width;
	var left_margin = 2;
	var top_margin = 14;
	var gap = 5;
	var space_for_the_untint_button = 10;
	var first_button_hOffset = left_edge + left_margin;
	new_game_button.vOffset = vertical_edge + top_margin;
	new_game_button.hOffset = first_button_hOffset;
	difficulty_button.vOffset = vertical_edge + top_margin;
	difficulty_button.hOffset = first_button_hOffset;
	undo_button.vOffset = vertical_edge + top_margin;
	undo_button.hOffset = first_button_hOffset;
	if (dim == 12){
		new_game_button.src = "";
		new_game_text.data = "";
		difficulty_button.src = "";
		difficulty_text.data = "";
	} else {
		new_game_button.src = "Resources/" + tile_texture + "/button.jpg";
		new_game_text.data = "New";
		new_game_button.hOffset = difficulty_button.hOffset + difficulty_button.width;
		new_game_text.setButtonText(new_game_button);
		difficulty_button.src = "Resources/" + tile_texture + "/button.jpg";
		difficulty_text.data = difficulty_abbr[difficulty];
		difficulty_text.setButtonText(difficulty_button);
		undo_button.hOffset = new_game_button.hOffset + new_game_button.width + gap;
	}
	undo_button.src = "Resources/" + tile_texture + "/button.jpg";
	if(undoList.length > 0){	
		undo_text.opacity=255;
	}else{
		undo_text.opacity=100;
	}
	undo_text.setButtonText(undo_button);

	hint_button.vOffset = vertical_edge + top_margin;
	hint_button.hOffset = undo_button.hOffset + undo_button.width + gap;
	hint_button.src = "Resources/" + tile_texture + "/button.jpg";
	hint_text.setButtonText(hint_button);
	if(no_help){	
		hint_text.opacity=100;
	}else{
		hint_text.opacity=255;
	}

	minimize_button.src = "Resources/" + tile_texture + "/button.jpg";
	maximize_button.src = "Resources/" + tile_texture + "/button.jpg";
	minimize_button.vOffset = vertical_edge + top_margin;
	minimize_button.hOffset = right_edge - left_margin - minimize_button.width;
	arrows.vOffset = minimize_button.vOffset + arrows.vRegistrationPoint;
	arrows.hOffset = minimize_button.hOffset + arrows.hRegistrationPoint;
	reset_button.vOffset = vertical_edge + top_margin;
	reset_button.hOffset = minimize_button.hOffset - gap - reset_button.width;
	reset_lid.vOffset = reset_button.vOffset;
	reset_lid.hOffset = reset_button.hOffset;
	
	var test_text = new Text();
	test_text.data = "Test";
	test_text.size = text1.size;
	test_text.style = text1.style;
	
	text1.vOffset = new_game_button.vOffset + new_game_button.height + gap + test_text.height;
	text1.hOffset = first_button_hOffset;
	text2.vOffset = text1.vOffset + test_text.height;
	text2.hOffset = text1.hOffset;
	if(border > 0){
		text1.bgOpacity = 255;
		text2.bgOpacity = 255;
	} else {
		text1.bgOpacity = 1;
		text2.bgOpacity = 1;
	}

} // end placeButtons()

function setScreenSize(){
	if (screen_size == "0"){  		//auto
		if (screen.availHeight < 800){
			size = SCREEN_SMALL;
		} else {
			size = SCREEN_LARGE;
		}
	} else{
		size = parseInt(screen_size,10);
	}
} // end setScreenSize()

function initializeData(){
	squares = new Array(dim);
	rows = new Array(dim);
	columns = new Array(dim);
	numbersR = new Array(dim);
	numbersC = new Array(dim);
	for (var x=0; x<dim; x++){
		squares[x] = new Array(dim);
		rows[x] = new Array(dim);
		columns[x] = new Array(dim);
		numbersR[x] = new Array(dim);
		numbersC[x] = new Array(dim);
		for (var y=0; y<dim; y++){
			squares[x][y] = new Array(2);
			rows[x][y] = new Array(2);
			columns[x][y] = new Array(2);
			numbersR[x][y] = new Array(2);
			numbersC[x][y] = new Array(2);
			squares[x][y][0] = Math.pow(2,dim)-1;
			squares[x][y][1] = dim;
			rows[x][y][0] = Math.pow(2,dim)-1;
			rows[x][y][1] = dim;
			columns[x][y][0] = Math.pow(2,dim)-1;
			columns[x][y][1] = dim;
			numbersR[x][y][0] = Math.pow(2,dim)-1;
			numbersR[x][y][1] = dim;
			numbersC[x][y][0] = Math.pow(2,dim)-1;
			numbersC[x][y][1] = dim;
		}
	}
}

function eraseData(){
	squares = new Array(dim);
	rows = new Array(dim);
	columns = new Array(dim);
	for (var x=0; x<dim; x++){
		squares[x] = new Array(dim);
		rows[x] = new Array(dim);
		columns[x] = new Array(dim);
		for (var y=0; y<dim; y++){
			squares[x][y] = new Array(2);
			rows[x][y] = new Array(2);
			columns[x][y] = new Array(2);
			squares[x][y][0] = 0;
			squares[x][y][1] = 0;
			rows[x][y][0] = 0;
			rows[x][y][1] = 0;
			columns[x][y][0] = 0;
			columns[x][y][1] = 0;
			numbersR[x][y][0] = 0;
			numbersR[x][y][1] = 0;
			numbersC[x][y][0] = 0;
			numbersC[x][y][1] = 0;
		}
	}
}

function initializeBoard(){
	setScreenSize();
	makeLookupTables();
	setFontSize();
	initializeData();
	tiles = new Array(dim);
	bigNumbers = new Array(dim);
	markupBigNumbers = new Array(dim);
	smallNumbers = new Array(dim);
	markupSmallNumbers = new Array(dim);
		
	var tile_shade;
	var tile_index = "";
	var light_tile_count = 0;
	var dark_tile_count = 0;
	var tile_z_order = frame_background.zOrder + 1;
				
	for (var i=0; i < dim; i++) {
		tiles[i] = new Array(dim);
		bigNumbers[i] = new Array(dim);
		smallNumbers[i] = new Array(dim);
		if(no_help){
			initializeMarkup(i);
		}
		for (var j=0; j < dim; j++) {
			tiles[i][j] = new Image();
			tiles[i][j].window = main_window;
			tiles[i][j].zOrder = tile_z_order;
			var square = tileToSquare[i][j];
			tile_shade = shades[square];
			if ((tile_texture != "plain") && (tile_texture != "generic")&& (tile_texture != "glass")){
				if(tile_shade == "light"){
					light_tile_count++;
					if (light_tile_count > 45){
						light_tile_count = 1;
					}
					tile_index = light_tile_count;
				}
				if (tile_shade == "dark"){
					dark_tile_count++;
					if (dark_tile_count > 36){
						dark_tile_count = 1;
					}
					tile_index =  dark_tile_count;
				}
			}
		
			tiles[i][j].src = "Resources/" + tile_texture + "/" + tile_shade + "_tile" + tile_index + ".jpg";
			tiles[i][j].height = tile_size[size];
			tiles[i][j].width = tile_size[size];
			tiles[i][j].hOffset = tileHorizOffset(j);
			tiles[i][j].vOffset = tileVertOffset(i);

			if (preferences.colorOn.value > 0){
				tiles[i][j].colorize = board_color;
			}else{
				tiles[i][j].colorize = "";
			}
			if (no_help){
				tiles[i][j].onMouseDown = "tileMouseDown("+i+","+j+");";
			}
		  
			bigNumbers[i][j] = new Text();
			bigNumbers[i][j].window = main_window;
			bigNumbers[i][j].hOffset = tiles[i][j].hOffset + tiles[i][j].height/2;
			bigNumbers[i][j].vOffset = tiles[i][j].vOffset + big_number_model.height;
			bigNumbers[i][j].hAlign = 'center';
			bigNumbers[i][j].size = big_font_size[size];
			bigNumbers[i][j].style = 'bold';
			bigNumbers[i][j].font = font;
			bigNumbers[i][j].style = fontStyle;
			bigNumbers[i][j].zOrder = tile_z_order + 1;
			if (no_help){
				bigNumbers[i][j].onMouseDown = "bigNumberClick("+i+","+j+");";
			}
			smallNumbers[i][j] = new Array(dim);
			for (var k=0; k < dim; k++){
				smallNumbers[i][j][k] = new Text();
				smallNumbers[i][j][k].window = main_window;
				smallNumbers[i][j][k].font = font;
				smallNumbers[i][j][k].style = fontStyle;
				smallNumbers[i][j][k].color = COLOR_NUMBERS;
				if (no_help){
					smallNumbers[i][j][k].data = "";
				} else {
					smallNumbers[i][j][k].data = candidates.charAt(k);
				}
				smallNumbers[i][j][k].size = small_font_size[size];
				smallNumbers[i][j][k].hOffset = smallNumberHorizOffset(i,j,k);
				smallNumbers[i][j][k].vOffset = smallNumberVertOffset(i,j,k);
				smallNumbers[i][j][k].hAlign = 'center';
				smallNumbers[i][j][k].onMouseDown = "smallNumberClick("+i+","+j+","+k+");";
				smallNumbers[i][j][k].zOrder = tile_z_order + 1;
				smallNumbers[i][j][k].onMouseEnter = "smallNumbersMouseEnter("+i+","+j+","+k+")";
				smallNumbers[i][j][k].onMouseExit = "smallNumbersMouseExit("+i+","+j+","+k+")";

				if(!tool_tips){
					smallNumbers[i][j][k].tooltip = "";
				} else {
					smallNumbers[i][j][k].tooltip = "Click to select. Shift-click to erase. Alt-click to color";
				}
			}
		}
	}
	resizeBackground();
	placeButtons();
	untint_button = null;
} // end initializeBoard()

function initializeMarkup(i){
	var dim_factor = Math.floor(Math.sqrt(dim));  // 9=3, 12=3, 16=4
	var dim_factor2 = Math.ceil(Math.sqrt(dim));  // 9=3, 12=4, 16=4
	markupBigNumbers[i] = new Text();
	markupBigNumbers[i].data = candidates.charAt(i);
	markupBigNumbers[i].window = markup_window;
	markupBigNumbers[i].size = 30;
	markupBigNumbers[i].font = font;
	markupBigNumbers[i].style = fontStyle;
	markupBigNumbers[i].hOffset = 10 + 30 * (i % dim_factor2);
	markupBigNumbers[i].vOffset = 30 + 30 * Math.floor((i / dim_factor2));
	markupBigNumbers[i].onMouseDown = "markupBigNumber(" + i + ")";

	 
	markupSmallNumbers[i] = new Text();
	markupSmallNumbers[i].data = candidates.charAt(i);
	markupSmallNumbers[i].window = markup_window;
	markupSmallNumbers[i].size = 20;
	markupSmallNumbers[i].font = font;
	markupSmallNumbers[i].style = fontStyle;
	markupSmallNumbers[i].color = COLOR_NUMBERS;
	markupSmallNumbers[i].hOffset = 3 + 20 * (i % dim_factor) + 130;
	markupSmallNumbers[i].vOffset = 25 + 20 * Math.floor((i / dim_factor2));
	markupSmallNumbers[i].onMouseDown = "markupSmallNumber(" + i + ")";
}

function changePreferences(){
	if (screen_size != preferences.screen_dimensions.value){
		screen_size = preferences.screen_dimensions.value;
		savePuzzle();
		boardSize(dim);
		restorePuzzle();
		hideHintText();
	}
	
	if (tile_texture != preferences.texture.value){
		tile_texture = preferences.texture.value;
		savePuzzle();
		initializeBoard();
		restorePuzzle();
		hideHintText();
	}

	if (board_color_on != preferences.colorOn.value || board_color != preferences.colorStyle.value){
		board_color_on = preferences.colorOn.value;
		board_color = preferences.colorStyle.value;
		if(preferences.colorOn.value > 0){
			colorizeTiles(board_color);
			colorizeFrame(board_color);
			colorizeButtons(board_color);
		} else {
			colorizeTiles("");
			colorizeFrame("");
			colorizeButtons("");
		}
	}
	
	if (pref_font != preferences.fontStyle.value){
		pref_font = preferences.fontStyle.value;
		font = pref_font.replace(/(^.)* (Regular.*)|(Plain.*)/,"$1");
		setFont(font);
	}
	
	if (bold_font != preferences.fontBold.value || italics_font != preferences.fontItalics.value){
		bold_font = preferences.fontBold.value;
		italics_font = preferences.fontItalics.value;
		if (bold_font > 0){
			if (italics_font > 0){
				fontStyle = "bold;italic";
			} else {
				fontStyle = "bold";
			}
		} else {
			if (italics_font > 0){
				fontStyle = "italic";
			} else {
				fontStyle = "";
			}
		}
		setFont(font);
	}
		
	tool_tips = (preferences.tooltip.value === "0");
	if(tool_tips){
		if(reset_buttonState != "open"){
			reset_button.tooltip = "Click to reset puzzle.";
			reset_lid.tooltip = "Click to close lid.";
		} else {
			reset_button.tooltip = "Click to open lid.";
			reset_lid.tooltip = "";
		}
		new_game_button.tooltip = "Click to generate a new puzzle.";
		difficulty_button.tooltip = "Click to change the difficulty of the next New puzzle.";
		hint_button.tooltip = "Click to get a hint on solving the puzzle.";
		undo_button.tooltip = "Click to undo your last move.";
		minimize_button.tooltip = "Click to minimize.";
	} else {
		reset_button.tooltip = "";
		reset_lid.tooltip = "";
		new_game_button.tooltip = "";
		difficulty_button.tooltip = "";
		hint_button.tooltip = "";
		undo_button.tooltip = "";
		minimize_button.tooltip = "";
	}
	for (var i=0; i < dim; i++) {
		for (var j=0; j < dim; j++) {
			for (var k=0; k<dim; k++) {
				if(!tool_tips){
					smallNumbers[i][j][k].tooltip = "";
				} else {
					smallNumbers[i][j][k].tooltip = "Click to select. Shift-click to erase. Alt-click to color";
				}
			}
		}	
	}
	
	if (border != preferences.border.value){
		border = preferences.border.value;		
		if(border > 0){
			opaceFrame(1);
			text1.bgOpacity = 255;
			text2.bgOpacity = 255;
		} else {
			opaceFrame(255);
			text1.bgOpacity = 1;
			text2.bgOpacity = 1;
		}
	}
	
	superhint = (preferences.superhint.value == "1");
	no_help = (preferences.no_help.value === "1");
	no_questions = (preferences.no_questions.value === "1");
	
	if((use_special_characters && preferences.special_character_set.value !== "1")||(!use_special_characters && preferences.special_character_set.value === "1")|| (preferences.character_set.value != special_character_set)){
		savePuzzle();
		use_special_characters = (preferences.special_character_set.value === "1");
		special_character_set = preferences.character_set.value;
		if(special_character_set.length != 16){
			special_character_set = default_candidates;
		}
		if(use_special_characters){
			candidates = special_character_set;	
		} else {
			candidates = default_candidates;
		}
		restorePuzzle();
	}	
} // end changePreferences

function eraseOldBoard(){
	tiles = null;
	bigNumbers = null;
	smallNumbers = null;
}

function boardSize(new_size){
	eraseOldBoard();
	dim = parseInt(new_size,10);
	preferences.board_size.value = dim;
	if (dim == 9){
		normalBoardDimensionsMenuItem.checked = true;
		largeBoardDimensionsMenuItem.checked = false;
		giantBoardDimensionsMenuItem.checked = false;
	}	
	if (dim == 12){
		normalBoardDimensionsMenuItem.checked = false;
		largeBoardDimensionsMenuItem.checked = true;
		giantBoardDimensionsMenuItem.checked = false;
	}
	if (dim == 16){
		normalBoardDimensionsMenuItem.checked = false;
		largeBoardDimensionsMenuItem.checked = false;
		giantBoardDimensionsMenuItem.checked = true;
	}
	initializeBoard();
}

function makeLookupTables(){
	tileToSquare = null;
	squareToTileI = null;
	squareToTileJ = null;
	
	switch (dim){
		case 12:
			tileToSquare = new Array(12);
			tileToSquare[0] = [0,0,0,0,1,1,1,1,2,2,2,2];
			tileToSquare[1] = [0,0,0,0,1,1,1,1,2,2,2,2];
			tileToSquare[2] = [0,0,0,0,1,1,1,1,2,2,2,2];
			tileToSquare[3] = [3,3,3,3,4,4,4,4,5,5,5,5];
			tileToSquare[4] = [3,3,3,3,4,4,4,4,5,5,5,5];
			tileToSquare[5] = [3,3,3,3,4,4,4,4,5,5,5,5];
			tileToSquare[6] = [6,6,6,6,7,7,7,7,8,8,8,8];
			tileToSquare[7] = [6,6,6,6,7,7,7,7,8,8,8,8];
			tileToSquare[8] = [6,6,6,6,7,7,7,7,8,8,8,8];
			tileToSquare[9] = [9,9,9,9,10,10,10,10,11,11,11,11];
			tileToSquare[10] = [9,9,9,9,10,10,10,10,11,11,11,11];
			tileToSquare[11] = [9,9,9,9,10,10,10,10,11,11,11,11];
			
			squareToTileI = new Array(12);
			squareToTileI[0] = [0,0,0,0,1,1,1,1,2,2,2,2];
			squareToTileI[1] = [0,0,0,0,1,1,1,1,2,2,2,2];
			squareToTileI[2] = [0,0,0,0,1,1,1,1,2,2,2,2];
			squareToTileI[3] = [3,3,3,3,4,4,4,4,5,5,5,5];
			squareToTileI[4] = [3,3,3,3,4,4,4,4,5,5,5,5];
			squareToTileI[5] = [3,3,3,3,4,4,4,4,5,5,5,5];
			squareToTileI[6] = [6,6,6,6,7,7,7,7,8,8,8,8];
			squareToTileI[7] = [6,6,6,6,7,7,7,7,8,8,8,8];
			squareToTileI[8] = [6,6,6,6,7,7,7,7,8,8,8,8];
			squareToTileI[9] = [9,9,9,9,10,10,10,10,11,11,11,11];
			squareToTileI[10] = [9,9,9,9,10,10,10,10,11,11,11,11];
			squareToTileI[11] = [9,9,9,9,10,10,10,10,11,11,11,11];
			
			squareToTileJ = new Array(12);
			squareToTileJ[0] = [0,1,2,3,0,1,2,3,0,1,2,3];
			squareToTileJ[1] = [4,5,6,7,4,5,6,7,4,5,6,7];
			squareToTileJ[2] = [8,9,10,11,8,9,10,11,8,9,10,11];
			squareToTileJ[3] = [0,1,2,3,0,1,2,3,0,1,2,3];
			squareToTileJ[4] = [4,5,6,7,4,5,6,7,4,5,6,7];
			squareToTileJ[5] = [8,9,10,11,8,9,10,11,8,9,10,11];
			squareToTileJ[6] = [0,1,2,3,0,1,2,3,0,1,2,3];
			squareToTileJ[7] = [4,5,6,7,4,5,6,7,4,5,6,7];
			squareToTileJ[8] = [8,9,10,11,8,9,10,11,8,9,10,11];
			squareToTileJ[9] = [0,1,2,3,0,1,2,3,0,1,2,3];
			squareToTileJ[10] = [4,5,6,7,4,5,6,7,4,5,6,7];
			squareToTileJ[11] = [8,9,10,11,8,9,10,11,8,9,10,11];

			shades = new Array(12);
			shades[0] = "light";
			shades[1] = "dark";
			shades[2] = "light";
			shades[3] = "dark";
			shades[4] = "light";
			shades[5] = "dark";
			shades[6] = "light";
			shades[7] = "dark";
			shades[8] = "light";
			shades[9] = "dark";
			shades[10] = "light";
			shades[11] = "dark";
	
			break;
			
		case 16:
			tileToSquare = new Array(16);
			tileToSquare[0] = [0,0,0,0,1,1,1,1,2,2,2,2,3,3,3,3];
			tileToSquare[1] = [0,0,0,0,1,1,1,1,2,2,2,2,3,3,3,3];
			tileToSquare[2] = [0,0,0,0,1,1,1,1,2,2,2,2,3,3,3,3];
			tileToSquare[3] = [0,0,0,0,1,1,1,1,2,2,2,2,3,3,3,3];
			tileToSquare[4] = [4,4,4,4,5,5,5,5,6,6,6,6,7,7,7,7];
			tileToSquare[5] = [4,4,4,4,5,5,5,5,6,6,6,6,7,7,7,7];
			tileToSquare[6] = [4,4,4,4,5,5,5,5,6,6,6,6,7,7,7,7];
			tileToSquare[7] = [4,4,4,4,5,5,5,5,6,6,6,6,7,7,7,7];
			tileToSquare[8] = [8,8,8,8,9,9,9,9,10,10,10,10,11,11,11,11];
			tileToSquare[9] = [8,8,8,8,9,9,9,9,10,10,10,10,11,11,11,11];
			tileToSquare[10] = [8,8,8,8,9,9,9,9,10,10,10,10,11,11,11,11];
			tileToSquare[11] = [8,8,8,8,9,9,9,9,10,10,10,10,11,11,11,11];
			tileToSquare[12] = [12,12,12,12,13,13,13,13,14,14,14,14,15,15,15,15];
			tileToSquare[13] = [12,12,12,12,13,13,13,13,14,14,14,14,15,15,15,15];
			tileToSquare[14] = [12,12,12,12,13,13,13,13,14,14,14,14,15,15,15,15];
			tileToSquare[15] = [12,12,12,12,13,13,13,13,14,14,14,14,15,15,15,15];
			
			squareToTileI = new Array(16);
			squareToTileI[0] = [0,0,0,0,1,1,1,1,2,2,2,2,3,3,3,3];
			squareToTileI[1] = [0,0,0,0,1,1,1,1,2,2,2,2,3,3,3,3];
			squareToTileI[2] = [0,0,0,0,1,1,1,1,2,2,2,2,3,3,3,3];
			squareToTileI[3] = [0,0,0,0,1,1,1,1,2,2,2,2,3,3,3,3];
			squareToTileI[4] = [4,4,4,4,5,5,5,5,6,6,6,6,7,7,7,7];
			squareToTileI[5] = [4,4,4,4,5,5,5,5,6,6,6,6,7,7,7,7];
			squareToTileI[6] = [4,4,4,4,5,5,5,5,6,6,6,6,7,7,7,7];
			squareToTileI[7] = [4,4,4,4,5,5,5,5,6,6,6,6,7,7,7,7];
			squareToTileI[8] = [8,8,8,8,9,9,9,9,10,10,10,10,11,11,11,11];
			squareToTileI[9] = [8,8,8,8,9,9,9,9,10,10,10,10,11,11,11,11];
			squareToTileI[10] = [8,8,8,8,9,9,9,9,10,10,10,10,11,11,11,11];
			squareToTileI[11] = [8,8,8,8,9,9,9,9,10,10,10,10,11,11,11,11];
			squareToTileI[12] = [12,12,12,12,13,13,13,13,14,14,14,14,15,15,15,15];
			squareToTileI[13] = [12,12,12,12,13,13,13,13,14,14,14,14,15,15,15,15];
			squareToTileI[14] = [12,12,12,12,13,13,13,13,14,14,14,14,15,15,15,15];
			squareToTileI[15] = [12,12,12,12,13,13,13,13,14,14,14,14,15,15,15,15];
			
			squareToTileJ = new Array(16);
			squareToTileJ[0] = [0,1,2,3,0,1,2,3,0,1,2,3,0,1,2,3];
			squareToTileJ[1] = [4,5,6,7,4,5,6,7,4,5,6,7,4,5,6,7];
			squareToTileJ[2] = [8,9,10,11,8,9,10,11,8,9,10,11,8,9,10,11];
			squareToTileJ[3] = [12,13,14,15,12,13,14,15,12,13,14,15,12,13,14,15];
			squareToTileJ[4] = [0,1,2,3,0,1,2,3,0,1,2,3,0,1,2,3];
			squareToTileJ[5] = [4,5,6,7,4,5,6,7,4,5,6,7,4,5,6,7];
			squareToTileJ[6] = [8,9,10,11,8,9,10,11,8,9,10,11,8,9,10,11];
			squareToTileJ[7] = [12,13,14,15,12,13,14,15,12,13,14,15,12,13,14,15];
			squareToTileJ[8] = [0,1,2,3,0,1,2,3,0,1,2,3,0,1,2,3];
			squareToTileJ[9] = [4,5,6,7,4,5,6,7,4,5,6,7,4,5,6,7];
			squareToTileJ[10] = [8,9,10,11,8,9,10,11,8,9,10,11,8,9,10,11];
			squareToTileJ[11] = [12,13,14,15,12,13,14,15,12,13,14,15,12,13,14,15];
			squareToTileJ[12] = [0,1,2,3,0,1,2,3,0,1,2,3,0,1,2,3];
			squareToTileJ[13] = [4,5,6,7,4,5,6,7,4,5,6,7,4,5,6,7];
			squareToTileJ[14] = [8,9,10,11,8,9,10,11,8,9,10,11,8,9,10,11];
			squareToTileJ[15] = [12,13,14,15,12,13,14,15,12,13,14,15,12,13,14,15];

			shades = new Array(16);
			shades[0] = "light";
			shades[1] = "dark";
			shades[2] = "light";
			shades[3] = "dark";
			shades[4] = "dark";
			shades[5] = "light";
			shades[6] = "dark";
			shades[7] = "light";
			shades[8] = "light";
			shades[9] = "dark";
			shades[10] = "light";
			shades[11] = "dark";
			shades[12] = "dark";
			shades[13] = "light";
			shades[14] = "dark";
			shades[15] = "light";

			break;
			
		case 9:
			tileToSquare = new Array(9);
			tileToSquare[0] = [0,0,0,1,1,1,2,2,2];
			tileToSquare[1] = [0,0,0,1,1,1,2,2,2];
			tileToSquare[2] = [0,0,0,1,1,1,2,2,2];
			tileToSquare[3] = [3,3,3,4,4,4,5,5,5];
			tileToSquare[4] = [3,3,3,4,4,4,5,5,5];
			tileToSquare[5] = [3,3,3,4,4,4,5,5,5];
			tileToSquare[6] = [6,6,6,7,7,7,8,8,8];
			tileToSquare[7] = [6,6,6,7,7,7,8,8,8];
			tileToSquare[8] = [6,6,6,7,7,7,8,8,8];
			
			squareToTileI = new Array(9);
			squareToTileI[0] = [0,0,0,1,1,1,2,2,2];
			squareToTileI[1] = [0,0,0,1,1,1,2,2,2];
			squareToTileI[2] = [0,0,0,1,1,1,2,2,2];
			squareToTileI[3] = [3,3,3,4,4,4,5,5,5];
			squareToTileI[4] = [3,3,3,4,4,4,5,5,5];
			squareToTileI[5] = [3,3,3,4,4,4,5,5,5];
			squareToTileI[6] = [6,6,6,7,7,7,8,8,8];
			squareToTileI[7] = [6,6,6,7,7,7,8,8,8];
			squareToTileI[8] = [6,6,6,7,7,7,8,8,8];
			
			squareToTileJ = new Array(9);
			squareToTileJ[0] = [0,1,2,0,1,2,0,1,2];
			squareToTileJ[1] = [3,4,5,3,4,5,3,4,5];
			squareToTileJ[2] = [6,7,8,6,7,8,6,7,8];
			squareToTileJ[3] = [0,1,2,0,1,2,0,1,2];
			squareToTileJ[4] = [3,4,5,3,4,5,3,4,5];
			squareToTileJ[5] = [6,7,8,6,7,8,6,7,8];
			squareToTileJ[6] = [0,1,2,0,1,2,0,1,2];
			squareToTileJ[7] = [3,4,5,3,4,5,3,4,5];
			squareToTileJ[8] = [6,7,8,6,7,8,6,7,8];
			
			shades = new Array(9);
			shades[0] = "light";
			shades[1] = "dark";
			shades[2] = "light";
			shades[3] = "dark";
			shades[4] = "light";
			shades[5] = "dark";
			shades[6] = "light";
			shades[7] = "dark";
			shades[8] = "light";

			break;
	} // end switch
}

function colorizeFrame(color){
	frame_background.colorize = color;
	frame_top.colorize = color;
	frame_bottom.colorize = color;
	frame_right.colorize = color;
	frame_left.colorize = color;
	frame_upper_left.colorize = color;
	frame_upper_right.colorize = color;
	frame_lower_left.colorize = color;
	frame_lower_right.colorize = color;
}

function colorizeButtons(color){
	undo_button.colorize = color;
	hint_button.colorize = color;
	new_game_button.colorize = color;
	difficulty_button.colorize = color;
	minimize_button.colorize = color;
	maximize_button.colorize = color;
}

function colorizeTiles(color){
	for (var i=0; i < dim; i++) {
		for (var j=0; j < dim; j++) {
			tiles[i][j].colorize = color;
		}
	}
}

function setPreferences(){
	if(preferences.colorOn.value > 0){
		colorizeTiles(board_color);
		colorizeFrame(board_color);
		colorizeButtons(board_color);
	} else {
		colorizeTiles("");
		colorizeFrame("");
		colorizeButtons("");
	}

	if(!tool_tips){
		hint_button.tooltip = "";
		reset_button.tooltip = "";
		new_game_button.tooltip = "";
		difficulty_button.tooltip = "";
		minimize_button.tooltip = "";
	} else {
		hint_button.tooltip = "Click to get a hint on solving the puzzle.";
		reset_button.tooltip = "Click to open lid.";
		new_game_button.tooltip = "Click to generate a new puzzle.";
		difficulty_button.tooltip = "Click to change the difficulty of the next New puzzle.";
		minimize_button.tooltip = "Click to minimize.";
	}


	if(preferences.border.value > 0){
		opaceFrame(1);
	} else {
		opaceFrame(255);
	}
	
	if (bold_font > 0){
		if (italics_font > 0){
			fontStyle = "bold;italic";
		} else {
			fontStyle = "bold";
		}
	} else {
		if (italics_font > 0){
			fontStyle = "italic";
		} else {
			fontStyle = "";
		}
	}
	setFont(font);
	
	if(use_special_characters){
		candidates = special_character_set;	
	} else {
		candidates = default_candidates;
	}
} // end setPreferences()

function opaceFrame(value){
		frame_top.opacity = value;
		frame_bottom.opacity = value;
		frame_upper_left.opacity = value;
		frame_upper_right.opacity = value;
		frame_left.opacity = value;
		frame_right.opacity = value;
		frame_lower_left.opacity = value;
		frame_lower_right.opacity = value;
		frame_background.opacity = value;
}

function setFontSize(){
	if (dim == 9) {
		small_font_size = [18, 18, 14];
	} else {
		if (dim == 12){
			small_font_size = [12, 12, 8];
		} else {
			small_font_size = [12, 12, 8];
		}
	}
	small_number_model.size = small_font_size[size];
	small_number_model.data = (dim==9?"9":"999");
	big_number_model.size = big_font_size[size];
	big_number_model.data = (dim==9?"9":"999");
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
*	
*														Misc
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
function viewRow(row){
	for(var k=0; k<dim; k++){
		var binary = rows[row][k][0].toString(2);
		while (binary.length < dim){
			binary = "0" + binary;
		}
		print (parseInt(k,10)+1 + ": " + binary + "  " + rows[row][k][1]);
	}
}
function viewColumn(column){
	for(var k=0; k<dim; k++){
		var binary = columns[column][k][0].toString(2);
		while (binary.length < dim){
			binary = "0" + binary;
		}
		print (parseInt(k,10)+1 + ": " + binary + "  " + columns[column][k][1]);
	}
}
function viewSquare(square){
	for(var k=0; k<dim; k++){
		var binary = squares[square][k][0].toString(2);
		while (binary.length < dim){
			binary = "0" + binary;
		}
		print (parseInt(k,10)+1 + ": " + binary + "  " + squares[square][k][1]);
	}
}
function viewNumberR(number){
	for(var k=0; k<dim; k++){
		var binary = numbersR[number][k][0].toString(2);
		while (binary.length < dim){
			binary = "0" + binary;
		}
		print (parseInt(k,10)+1 + ": " + binary + "  " + numbersR[number][k][1]);
	}
}
function viewNumberC(number){
	for(var k=0; k<dim; k++){
		var binary = numbersC[number][k][0].toString(2);
		while (binary.length < dim){
			binary = "0" + binary;
		}
		print (parseInt(k,10)+1 + ": " + binary + "  " + numbersC[number][k][1]);
	}
}

function factorial(n){
	if (n <= 1) return 1;
	return n*factorial(n-1);
}

function combination(r,n){
	return (factorial(n)/(factorial(n-r)*factorial(r)));
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
*	
*														Main
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
changeSolveLevel(solve_level);
boardSize(preferences.board_size.value);
setPreferences();
restorePuzzle();
restoreRecent();
